@@ -530,11 +530,11 @@ func (s *sealSuite) TestResealKeyToModeenvWithSystemFallback(c *C) {
 
 		// set mock key resealing
 		resealKeysCalls := 0
-		restore = boot.MockResealKeyForBootChains(func(unlocker boot.Unlocker, method device.SealingMethod, rootdirArg string, params *boot.ResealKeyForBootChainsParams, expectReseal bool) error {
+		restore = boot.MockResealKeyForBootChains(func(unlocker boot.Unlocker, method device.SealingMethod, rootdirArg string, params *boot.ResealKeyForBootChainsParams) error {
 			resealKeysCalls++
 
 			c.Check(method, Equals, device.SealingMethodTPM)
-			c.Check(expectReseal, Equals, false)
+			c.Check(params.Options.ExpectReseal, Equals, false)
 			c.Check(rootdirArg, Equals, rootdir)
 
 			c.Check(params.RunModeBootChains, DeepEquals, expectedRunBootChains)
@@ -545,8 +545,8 @@ func (s *sealSuite) TestResealKeyToModeenvWithSystemFallback(c *C) {
 		})
 		defer restore()
 
-		const expectReseal = false
-		err = boot.ResealKeyToModeenv(rootdir, modeenv, expectReseal, nil)
+		opts := boot.ResealKeyToModeenvOptions{ExpectReseal: false}
+		err = boot.ResealKeyToModeenv(rootdir, modeenv, opts, nil)
 		if !tc.sealedKeys {
 			// did nothing
 			c.Assert(err, IsNil)
@@ -624,11 +624,11 @@ func (s *sealSuite) TestResealKeyToModeenvRecoveryKeysForGoodSystemsOnly(c *C) {
 
 	// set mock key resealing
 	resealKeysCalls := 0
-	restore = boot.MockResealKeyForBootChains(func(unlocker boot.Unlocker, method device.SealingMethod, rootdirArg string, params *boot.ResealKeyForBootChainsParams, expectReseal bool) error {
+	restore = boot.MockResealKeyForBootChains(func(unlocker boot.Unlocker, method device.SealingMethod, rootdirArg string, params *boot.ResealKeyForBootChainsParams) error {
 		resealKeysCalls++
 
 		c.Check(method, Equals, device.SealingMethodTPM)
-		c.Check(expectReseal, Equals, false)
+		c.Check(params.Options.ExpectReseal, Equals, false)
 		c.Check(rootdirArg, Equals, rootdir)
 
 		c.Assert(resealKeysCalls, Equals, 1)
@@ -769,8 +769,8 @@ func (s *sealSuite) TestResealKeyToModeenvRecoveryKeysForGoodSystemsOnly(c *C) {
 	// expectReseal to false as it doesn't matter;
 	// the behavior with unasserted kernel is tested in
 	// boot_test.go specific tests
-	const expectReseal = false
-	err = boot.ResealKeyToModeenv(rootdir, modeenv, expectReseal, nil)
+	opts := boot.ResealKeyToModeenvOptions{ExpectReseal: false}
+	err = boot.ResealKeyToModeenv(rootdir, modeenv, opts, nil)
 	c.Assert(err, IsNil)
 	c.Assert(resealKeysCalls, Equals, 1)
 }
@@ -839,10 +839,10 @@ func (s *sealSuite) TestResealKeyToModeenvFallbackCmdline(c *C) {
 
 	// set mock key resealing
 	resealKeysCalls := 0
-	restore = boot.MockResealKeyForBootChains(func(unlocker boot.Unlocker, method device.SealingMethod, rootdirArg string, params *boot.ResealKeyForBootChainsParams, expectReseal bool) error {
+	restore = boot.MockResealKeyForBootChains(func(unlocker boot.Unlocker, method device.SealingMethod, rootdirArg string, params *boot.ResealKeyForBootChainsParams) error {
 		c.Check(rootdirArg, Equals, rootdir)
 		c.Check(method, Equals, device.SealingMethodTPM)
-		c.Check(expectReseal, Equals, false)
+		c.Check(params.Options.ExpectReseal, Equals, false)
 
 		resealKeysCalls++
 		c.Logf("reseal: %+v", params)
@@ -918,8 +918,8 @@ func (s *sealSuite) TestResealKeyToModeenvFallbackCmdline(c *C) {
 	})
 	defer restore()
 
-	const expectReseal = false
-	err = boot.ResealKeyToModeenv(rootdir, modeenv, expectReseal, nil)
+	opts := boot.ResealKeyToModeenvOptions{ExpectReseal: false}
+	err = boot.ResealKeyToModeenv(rootdir, modeenv, opts, nil)
 	c.Assert(err, IsNil)
 	c.Assert(resealKeysCalls, Equals, 1)
 }
@@ -1317,7 +1317,7 @@ func (s *sealSuite) TestRecoveryBootChainsForSystems(c *C) {
 			case "20200831":
 				kernelRev = 3
 			case "off-model":
-				systemModel = boottest.MakeMockUC20Model(map[string]interface{}{
+				systemModel = boottest.MakeMockUC20Model(map[string]any{
 					"model": "model-mismatch-uc20",
 				})
 			default:
@@ -1414,7 +1414,7 @@ func (s *sealSuite) TestSealKeyModelParams(c *C) {
 		"loader-loader-hash2",
 	})
 
-	oldmodel := boottest.MakeMockUC20Model(map[string]interface{}{
+	oldmodel := boottest.MakeMockUC20Model(map[string]any{
 		"model":     "old-model-uc20",
 		"timestamp": "2019-10-01T08:00:00+00:00",
 	})
@@ -1684,10 +1684,10 @@ func (s *sealSuite) TestResealKeyToModeenvWithFdeHookCalled(c *C) {
 	defer dirs.SetRootDir("")
 
 	mockResealKeyForBootChainsCalls := 0
-	restore := boot.MockResealKeyForBootChains(func(unlocker boot.Unlocker, method device.SealingMethod, rootdirArg string, params *boot.ResealKeyForBootChainsParams, expectReseal bool) error {
+	restore := boot.MockResealKeyForBootChains(func(unlocker boot.Unlocker, method device.SealingMethod, rootdirArg string, params *boot.ResealKeyForBootChainsParams) error {
 		c.Check(rootdirArg, Equals, rootdir)
 		c.Check(method, Equals, device.SealingMethodFDESetupHook)
-		c.Check(expectReseal, Equals, false)
+		c.Check(params.Options.ExpectReseal, Equals, false)
 
 		mockResealKeyForBootChainsCalls++
 		return nil
@@ -1718,8 +1718,8 @@ func (s *sealSuite) TestResealKeyToModeenvWithFdeHookCalled(c *C) {
 		Grade:          string(model.Grade()),
 		ModelSignKeyID: model.SignKeyID(),
 	}
-	expectReseal := false
-	err = boot.ResealKeyToModeenv(rootdir, modeenv, expectReseal, nil)
+	opts := boot.ResealKeyToModeenvOptions{ExpectReseal: false}
+	err = boot.ResealKeyToModeenv(rootdir, modeenv, opts, nil)
 	c.Assert(err, IsNil)
 	c.Check(mockResealKeyForBootChainsCalls, Equals, 1)
 }
@@ -1730,10 +1730,10 @@ func (s *sealSuite) TestResealKeyToModeenvWithFdeHookVerySad(c *C) {
 	defer dirs.SetRootDir("")
 
 	mockResealKeyForBootChainsCalls := 0
-	restore := boot.MockResealKeyForBootChains(func(unlocker boot.Unlocker, method device.SealingMethod, rootdirArg string, params *boot.ResealKeyForBootChainsParams, expectReseal bool) error {
+	restore := boot.MockResealKeyForBootChains(func(unlocker boot.Unlocker, method device.SealingMethod, rootdirArg string, params *boot.ResealKeyForBootChainsParams) error {
 		c.Check(rootdirArg, Equals, rootdir)
 		c.Check(method, Equals, device.SealingMethodFDESetupHook)
-		c.Check(expectReseal, Equals, false)
+		c.Check(params.Options.ExpectReseal, Equals, false)
 		mockResealKeyForBootChainsCalls++
 		return fmt.Errorf("fde setup hook failed")
 	})
@@ -1755,8 +1755,8 @@ func (s *sealSuite) TestResealKeyToModeenvWithFdeHookVerySad(c *C) {
 		Grade:          string(model.Grade()),
 		ModelSignKeyID: model.SignKeyID(),
 	}
-	expectReseal := false
-	err = boot.ResealKeyToModeenv(rootdir, modeenv, expectReseal, nil)
+	opts := boot.ResealKeyToModeenvOptions{ExpectReseal: false}
+	err = boot.ResealKeyToModeenv(rootdir, modeenv, opts, nil)
 	c.Assert(err, ErrorMatches, "fde setup hook failed")
 	c.Check(mockResealKeyForBootChainsCalls, Equals, 1)
 }
@@ -1778,7 +1778,7 @@ func (s *sealSuite) testResealKeyToModeenvWithTryModel(c *C, shimId, grubId stri
 
 	model := boottest.MakeMockUC20Model()
 	// a try model which would normally only appear during remodel
-	tryModel := boottest.MakeMockUC20Model(map[string]interface{}{
+	tryModel := boottest.MakeMockUC20Model(map[string]any{
 		"model": "try-my-model-uc20",
 		"grade": "secured",
 	})
@@ -1829,7 +1829,7 @@ func (s *sealSuite) testResealKeyToModeenvWithTryModel(c *C, shimId, grubId stri
 		}
 		if label == "off-model" {
 			// a model that matches neither current not try models
-			systemModel = boottest.MakeMockUC20Model(map[string]interface{}{
+			systemModel = boottest.MakeMockUC20Model(map[string]any{
 				"model": "different-model-uc20",
 				"grade": "secured",
 			})
@@ -1842,10 +1842,10 @@ func (s *sealSuite) testResealKeyToModeenvWithTryModel(c *C, shimId, grubId stri
 
 	// set mock key resealing
 	resealKeysCalls := 0
-	restore = boot.MockResealKeyForBootChains(func(unlocker boot.Unlocker, method device.SealingMethod, rootdirArg string, params *boot.ResealKeyForBootChainsParams, expectReseal bool) error {
+	restore = boot.MockResealKeyForBootChains(func(unlocker boot.Unlocker, method device.SealingMethod, rootdirArg string, params *boot.ResealKeyForBootChainsParams) error {
 		c.Check(rootdirArg, Equals, rootdir)
 		c.Check(method, Equals, device.SealingMethodTPM)
-		c.Check(expectReseal, Equals, false)
+		c.Check(params.Options.ExpectReseal, Equals, false)
 		resealKeysCalls++
 
 		kernelOldRecovery := bootloader.NewBootFile("/var/lib/snapd/seed/snaps/pc-kernel_1.snap", "kernel.efi", bootloader.RoleRecovery)
@@ -1963,8 +1963,8 @@ func (s *sealSuite) testResealKeyToModeenvWithTryModel(c *C, shimId, grubId stri
 	// expectReseal to false as it doesn't matter;
 	// the behavior with unasserted kernel is tested in
 	// boot_test.go specific tests
-	const expectReseal = false
-	err = boot.ResealKeyToModeenv(rootdir, modeenv, expectReseal, nil)
+	opts := boot.ResealKeyToModeenvOptions{ExpectReseal: false}
+	err = boot.ResealKeyToModeenv(rootdir, modeenv, opts, nil)
 	c.Assert(err, IsNil)
 	c.Assert(resealKeysCalls, Equals, 1)
 }
@@ -2029,14 +2029,14 @@ func (s *sealSuite) TestWithBootChains(c *C) {
 	})
 	defer restore()
 
-	var chains *boot.ResealKeyForBootChainsParams
-	err = boot.WithBootChains(func(ch *boot.ResealKeyForBootChainsParams) error {
+	var chains boot.BootChains
+	err = boot.WithBootChains(func(ch boot.BootChains) error {
 		chains = ch
 		return nil
 	}, device.SealingMethodTPM)
 	c.Assert(err, IsNil)
 
-	c.Check(chains, DeepEquals, &boot.ResealKeyForBootChainsParams{
+	c.Check(chains, DeepEquals, boot.BootChains{
 		RunModeBootChains: []boot.BootChain{
 			{
 				BrandID:        "my-brand",
