@@ -19,11 +19,30 @@
 
 package assets
 
-func init() {
-	registerSnippetForEditions("grub.cfg:static-cmdline", []ForEditions{
+import (
+	"github.com/snapcore/snapd/arch"
+)
+
+var cmdlineForArch = map[string][]ForEditions{
+	"amd64": {
 		{FirstEdition: 1, Snippet: []byte("console=ttyS0 console=tty1 panic=-1")},
-	})
-	registerSnippetForEditions("grub-recovery.cfg:static-cmdline", []ForEditions{
+		{FirstEdition: 3, Snippet: []byte("console=ttyS0,115200n8 console=tty1 panic=-1")},
+	},
+	"arm64": {
+		{FirstEdition: 1, Snippet: []byte("panic=-1")},
+	},
+	"i386": {
 		{FirstEdition: 1, Snippet: []byte("console=ttyS0 console=tty1 panic=-1")},
-	})
+		{FirstEdition: 3, Snippet: []byte("console=ttyS0,115200n8 console=tty1 panic=-1")},
+	},
+}
+
+func registerGrubSnippets() {
+	snippets := cmdlineForArch[arch.DpkgArchitecture()]
+	registerSnippetForEditions("grub.cfg:static-cmdline", snippets)
+	registerSnippetForEditions("grub-recovery.cfg:static-cmdline", snippets)
+}
+
+func init() {
+	registerGrubSnippets()
 }
