@@ -46,13 +46,13 @@
  * Error structure.
  **/
 typedef struct sc_error {
-	// Error domain defines a scope for particular error codes.
-	const char *domain;
-	// Code differentiates particular errors for the programmer.
-	// The code may be zero if the particular meaning is not relevant.
-	int code;
-	// Message carries a formatted description of the problem.
-	char *msg;
+    // Error domain defines a scope for particular error codes.
+    const char *domain;
+    // Code differentiates particular errors for the programmer.
+    // The code may be zero if the particular meaning is not relevant.
+    int code;
+    // Message carries a formatted description of the problem.
+    char *msg;
 } sc_error;
 
 /**
@@ -67,12 +67,12 @@ typedef struct sc_error {
 
 /** sc_libsnap_error represents distinct error codes used by libsnap-confine-private library. */
 typedef enum sc_libsnap_error {
-	/** SC_UNSPECIFIED_ERROR indicates an error not worthy of a distinct code. */
-	SC_UNSPECIFIED_ERROR = 0,
-	/** SC_API_MISUSE indicates that public API was called incorrectly. */
-	SC_API_MISUSE,
-	/** SC_BUG indicates that private API was called incorrectly. */
-	SC_BUG,
+    /** SC_UNSPECIFIED_ERROR indicates an error not worthy of a distinct code. */
+    SC_UNSPECIFIED_ERROR = 0,
+    /** SC_API_MISUSE indicates that public API was called incorrectly. */
+    SC_API_MISUSE,
+    /** SC_BUG indicates that private API was called incorrectly. */
+    SC_BUG,
 } sc_libsnap_error;
 
 /**
@@ -85,9 +85,8 @@ typedef enum sc_libsnap_error {
  *
  * This function calls die() in case of memory allocation failure.
  **/
-__attribute__((warn_unused_result,
-	       format(printf, 3, 4) SC_APPEND_RETURNS_NONNULL))
-sc_error *sc_error_init(const char *domain, int code, const char *msgfmt, ...);
+__attribute__((warn_unused_result, format(printf, 3, 4) SC_APPEND_RETURNS_NONNULL)) sc_error *sc_error_init(
+    const char *domain, int code, const char *msgfmt, ...);
 
 /**
  * Initialize an unspecified error with formatted message.
@@ -95,9 +94,8 @@ sc_error *sc_error_init(const char *domain, int code, const char *msgfmt, ...);
  * This is just syntactic sugar for sc_error_init(SC_LIBSNAP_ERROR,
  * SC_UNSPECIFIED_ERROR, msgfmt, ...) which is repeated often.
  **/
-__attribute__((warn_unused_result,
-	       format(printf, 1, 2) SC_APPEND_RETURNS_NONNULL))
-sc_error *sc_error_init_simple(const char *msgfmt, ...);
+__attribute__((warn_unused_result, format(printf, 1, 2) SC_APPEND_RETURNS_NONNULL)) sc_error *sc_error_init_simple(
+    const char *msgfmt, ...);
 
 /**
  * Initialize an API misuse error with formatted message.
@@ -105,9 +103,8 @@ sc_error *sc_error_init_simple(const char *msgfmt, ...);
  * This is just syntactic sugar for sc_error_init(SC_LIBSNAP_DOMAIN,
  * SC_API_MISUSE, msgfmt, ...) which is repeated often.
  **/
-__attribute__((warn_unused_result,
-	       format(printf, 1, 2) SC_APPEND_RETURNS_NONNULL))
-sc_error *sc_error_init_api_misuse(const char *msgfmt, ...);
+__attribute__((warn_unused_result, format(printf, 1, 2) SC_APPEND_RETURNS_NONNULL)) sc_error *sc_error_init_api_misuse(
+    const char *msgfmt, ...);
 
 /**
  * Initialize an errno-based error.
@@ -117,9 +114,8 @@ sc_error *sc_error_init_api_misuse(const char *msgfmt, ...);
  *
  * This function calls die() in case of memory allocation failure.
  **/
-__attribute__((warn_unused_result,
-	       format(printf, 2, 3) SC_APPEND_RETURNS_NONNULL))
-sc_error *sc_error_init_from_errno(int errno_copy, const char *msgfmt, ...);
+__attribute__((warn_unused_result, format(printf, 2, 3) SC_APPEND_RETURNS_NONNULL)) sc_error *sc_error_init_from_errno(
+    int errno_copy, const char *msgfmt, ...);
 
 /**
  * Get the error domain out of an error object.
@@ -127,8 +123,7 @@ sc_error *sc_error_init_from_errno(int errno_copy, const char *msgfmt, ...);
  * The error domain acts as a namespace for error codes.
  * No change of ownership takes place.
  **/
-__attribute__((warn_unused_result SC_APPEND_RETURNS_NONNULL))
-const char *sc_error_domain(sc_error * err);
+__attribute__((warn_unused_result SC_APPEND_RETURNS_NONNULL)) const char *sc_error_domain(sc_error *err);
 
 /**
  * Get the error code out of an error object.
@@ -140,8 +135,7 @@ const char *sc_error_domain(sc_error * err);
  * can rely on programmatically. This can be used to return an error message
  * without having to allocate a distinct code for each one.
  **/
-__attribute__((warn_unused_result))
-int sc_error_code(sc_error * err);
+__attribute__((warn_unused_result)) int sc_error_code(sc_error *err);
 
 /**
  * Get the error message out of an error object.
@@ -149,15 +143,14 @@ int sc_error_code(sc_error * err);
  * The error message is bound to the life-cycle of the error object.
  * No change of ownership takes place.
  **/
-__attribute__((warn_unused_result SC_APPEND_RETURNS_NONNULL))
-const char *sc_error_msg(sc_error * err);
+__attribute__((warn_unused_result SC_APPEND_RETURNS_NONNULL)) const char *sc_error_msg(sc_error *err);
 
 /**
  * Free an error object.
  *
  * The error object can be NULL.
  **/
-void sc_error_free(sc_error * error);
+void sc_error_free(sc_error *error);
 
 /**
  * Cleanup an error with sc_error_free()
@@ -165,8 +158,7 @@ void sc_error_free(sc_error * error);
  * This function is designed to be used with
  * __attribute__((cleanup(sc_cleanup_error))).
  **/
-__attribute__((nonnull))
-void sc_cleanup_error(sc_error ** ptr);
+__attribute__((nonnull)) void sc_cleanup_error(sc_error **ptr);
 
 /**
  *
@@ -177,7 +169,7 @@ void sc_cleanup_error(sc_error ** ptr);
  * The error message is derived from the data in the error, using the special
  * errno domain to provide additional information if that is available.
  **/
-void sc_die_on_error(sc_error * error);
+void sc_die_on_error(sc_error *error);
 
 /**
  * Forward an error to the caller.
@@ -194,7 +186,7 @@ void sc_die_on_error(sc_error * error);
  **/
 // NOTE: There's no nonnull(1) attribute as the recipient *can* be NULL. With
 // the attribute in place GCC optimizes some things out and tests fail.
-int sc_error_forward(sc_error ** recipient, sc_error * error);
+int sc_error_forward(sc_error **recipient, sc_error *error);
 
 /**
  * Check if a given error matches the specified domain and code.
@@ -202,7 +194,6 @@ int sc_error_forward(sc_error ** recipient, sc_error * error);
  * It is okay to match a NULL error, the function simply returns false in that
  * case. The domain cannot be NULL though.
  **/
-__attribute__((warn_unused_result))
-bool sc_error_match(sc_error * error, const char *domain, int code);
+__attribute__((warn_unused_result)) bool sc_error_match(sc_error *error, const char *domain, int code);
 
 #endif
