@@ -21,6 +21,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <glob.h>
+#include <inttypes.h>
 #include <sched.h>
 #include <signal.h>
 #include <stdbool.h>
@@ -41,6 +42,7 @@
 #include "../libsnap-confine-private/feature.h"
 #include "../libsnap-confine-private/infofile.h"
 #include "../libsnap-confine-private/locking.h"
+#include "../libsnap-confine-private/privs.h"
 #include "../libsnap-confine-private/secure-getenv.h"
 #include "../libsnap-confine-private/snap-dir.h"
 #include "../libsnap-confine-private/snap.h"
@@ -48,6 +50,7 @@
 #include "../libsnap-confine-private/tool.h"
 #include "../libsnap-confine-private/utils.h"
 #include "cookie-support.h"
+#include "group-policy.h"
 #include "mount-support.h"
 #include "ns-support.h"
 #include "seccomp-support.h"
@@ -254,12 +257,9 @@ the_void:
      * created on demand. */
     void_dir_fd = open(sc_void_dir, O_DIRECTORY | O_PATH | O_NOFOLLOW | O_CLOEXEC);
     if (void_dir_fd < 0 && errno == ENOENT) {
-        if (mkdir(sc_void_dir, 0111) < 0) {
+        if (sc_ensure_mkdir(sc_void_dir, 0111, 0, 0) != 0) {
             die("cannot create void directory: %s", sc_void_dir);
         }
-        if (lchown(sc_void_dir, 0, 0) < 0) {
-            die("cannot change ownership of void directory %s", sc_void_dir);
-        }
         void_dir_fd = open(sc_void_dir, O_DIRECTORY | O_PATH | O_NOFOLLOW | O_CLOEXEC);
     }
     if (void_dir_fd < 0) {
@@ -277,8 +277,8 @@ static void log_startup_stage(const char *stage) {
     }
     struct timeval tv;
     gettimeofday(&tv, NULL);
-    debug("-- snap startup {\"stage\":\"%s\", \"time\":\"%lld.%06lld\"}", stage, (long long int)tv.tv_sec,
-          (long long int)tv.tv_usec);
+    debug("-- snap startup {\"stage\":\"%s\", \"time\":\"%" PRId64 ".%06" PRId64 "\"}", stage, (int64_t)tv.tv_sec,
+          (int64_t)tv.tv_usec);
 }
 
 /**
@@ -296,12 +296,7 @@ int main(int argc, char **argv) {
     sc_error *err = NULL;
 
     log_startup_stage("snap-confine enter");
-
-    // Figure out what is the SNAP_MOUNT_DIR in practice.
-    sc_probe_snap_mount_dir_from_pid_1_mount_ns(AT_FDCWD, &err);
-    sc_die_on_error(err);
-
-    debug("SNAP_MOUNT_DIR (probed): %s", sc_snap_mount_dir(NULL));
+    sc_debug_capabilities("caps at startup");
 
     // Use our super-defensive parser to figure out what we've been asked to do.
     struct sc_args *args SC_CLEANUP(sc_cleanup_args) = NULL;
@@ -310,11 +305,6 @@ int main(int argc, char **argv) {
     args = sc_nonfatal_parse_args(&argc, &argv, &err);
     sc_die_on_error(err);
 
-    // Remember certain properties of the process that are clobbered by
-    // snap-confine during execution. Those are restored just before calling
-    // execv.
-    sc_preserve_and_sanitize_process_state(&proc_state);
-
     // We've been asked to print the version string so let's just do that.
     if (sc_args_is_version_query(args)) {
         printf("%s %s\n", PACKAGE, PACKAGE_VERSION);
@@ -333,8 +323,6 @@ int main(int argc, char **argv) {
     // should be prepared to handle NULL.
     const char *snap_component_name_env = getenv("SNAP_COMPONENT_NAME");
 
-    sc_init_invocation(&invocation, args, snap_instance_name_env, snap_component_name_env);
-
     // Who are we?
     uid_t real_uid, effective_uid, saved_uid;
     gid_t real_gid, effective_gid, saved_gid;
@@ -347,11 +335,121 @@ int main(int argc, char **argv) {
     debug("ruid: %d, euid: %d, suid: %d", real_uid, effective_uid, saved_uid);
     debug("rgid: %d, egid: %d, sgid: %d", real_gid, effective_gid, saved_gid);
 
-    // snap-confine needs to run as root for cgroup/udev/mount/apparmor/etc setup.
-    if (effective_uid != 0) {
-        die("need to run as root or suid");
+    struct sc_apparmor apparmor;
+    sc_init_apparmor_support(&apparmor);
+    if (!apparmor.is_confined && apparmor.mode != SC_AA_NOT_APPLICABLE && real_uid != 0) {
+        // Refuse to run when this process is running unconfined on a system
+        // that supports AppArmor when the effective uid is root and the real
+        // id is non-root.  This protects against, for example, unprivileged
+        // users trying to leverage the snap-confine in the core snap to
+        // escalate privileges.
+        errno = 0;  // errno is insignificant here
+        die("snap-confine has elevated permissions and is not confined"
+            " but should be. Refusing to continue to avoid"
+            " permission escalation attacks\n"
+            "Please make sure that the snapd.apparmor service is enabled and started.");
     }
 
+    sc_debug_capabilities("initial caps");
+
+    static const cap_value_t snap_confine_caps[] = {
+        CAP_DAC_OVERRIDE,     // poking around as a regular user
+        CAP_DAC_READ_SEARCH,  // same as above
+        CAP_SYS_ADMIN,        // mounts, unshare
+        CAP_SYS_CHROOT,       // pivot_root into a new root
+        CAP_CHOWN,            // file ownership
+        CAP_FOWNER,           // to create tmp dir with sticky bit
+        CAP_SYS_PTRACE,       // to inspect the mount namespace of PID1
+    };
+
+    /* We may be invoking tools such as snap-update-ns or snap-discard which are
+     * executed in a forked process, the child can inherit at most these
+     * capabilities */
+    static const cap_value_t helper_tools_inheritable_caps[] = {
+        CAP_DAC_OVERRIDE,  // poking around as a regular user
+        CAP_SYS_ADMIN,     // mounts
+        CAP_CHOWN,         // file ownership
+    };
+
+    /* Capability setup:
+     * 1. Permitted caps are obtained from file.
+     * 2. Restore those capabilities that we really need into the
+     *    "effective" set.
+     * 3. Capabilities needed by either us or by any of our child processes
+     *    need to be set into the "permitted" set.
+     * 4. Capabilities needed by our helper child processes need to be set
+     *    into the "permitted", "inheritable" and "ambient" sets.
+     *
+     * Before executing the snap application we'll drop all capabilities.
+     */
+
+    /* Set of caps for executing privileged operations. */
+    cap_t caps_privileged SC_CLEANUP(sc_cleanup_cap_t) = cap_get_proc();
+    if (caps_privileged == NULL) {
+        die("cannot obtain current caps");
+    }
+
+    if (cap_set_flag(caps_privileged, CAP_EFFECTIVE, SC_ARRAY_SIZE(snap_confine_caps), snap_confine_caps, CAP_SET) !=
+        0) {
+        die("cannot fill effective capability set");
+    }
+    /* inheritable caps when forking off to a helper tool */
+    if (cap_set_flag(caps_privileged, CAP_INHERITABLE, SC_ARRAY_SIZE(helper_tools_inheritable_caps),
+                     helper_tools_inheritable_caps, CAP_SET) != 0) {
+        die("cannot fill inheritable capability set");
+    }
+
+    /* Set of caps we use while not performing any privileged operations which
+     * require CAP_SYS_ADMIN, but also keep CAP_DAC_OVERRIDE in case we need to
+     * restore it ater, if the user was really root */
+    static const cap_value_t keep_no_effective_caps[] = {
+        CAP_SYS_ADMIN, /* seccomp */
+        CAP_DAC_OVERRIDE,
+    };
+
+    cap_t caps_no_effective SC_CLEANUP(sc_cleanup_cap_t) = cap_init();
+    if (caps_no_effective == NULL) {
+        die("cannot copy caps");
+    }
+
+    if (cap_set_flag(caps_no_effective, CAP_PERMITTED, SC_ARRAY_SIZE(keep_no_effective_caps), keep_no_effective_caps,
+                     CAP_SET) != 0) {
+        die("cannot set capapbility flags");
+    }
+
+    /* set privileged capabilities */
+    if (cap_set_proc(caps_privileged) != 0) {
+        die("cannot set capabilities");
+    }
+
+    sc_debug_capabilities("after setting privileged caps");
+
+    /* reset ambient caps, those are set accordingly depening on the
+     * requirements of a specific tool */
+    if (sc_cap_reset_ambient() != 0) {
+        die("cannot reset ambient capabilities");
+    }
+
+    /* Some distributions choose to limit the ability of regular users to run
+     * snaps to members of a specific local group. Note we need to keep elevated
+     * privileges as the code peeks into pid 1 root filesystem to locate
+     * snap-confine */
+    sc_assert_host_local_group_policy(AT_FDCWD, &err);
+    sc_die_on_error(err);
+
+    // Figure out what is the SNAP_MOUNT_DIR in practice.
+    sc_probe_snap_mount_dir_from_pid_1_mount_ns(AT_FDCWD, &err);
+    sc_die_on_error(err);
+
+    debug("SNAP_MOUNT_DIR (probed): %s", sc_snap_mount_dir(NULL));
+
+    sc_init_invocation(&invocation, args, snap_instance_name_env, snap_component_name_env);
+
+    // Remember certain properties of the process that are clobbered by
+    // snap-confine during execution. Those are restored just before calling
+    // execv.
+    sc_preserve_and_sanitize_process_state(&proc_state);
+
     char *snap_context SC_CLEANUP(sc_cleanup_string) = NULL;
     // Do no get snap context value if running a hook (we don't want to overwrite hook's SNAP_COOKIE)
     if (!sc_is_hook_security_tag(invocation.security_tag)) {
@@ -366,21 +464,6 @@ int main(int argc, char **argv) {
          * information is collected but discarded. */
     }
 
-    struct sc_apparmor apparmor;
-    sc_init_apparmor_support(&apparmor);
-    if (!apparmor.is_confined && apparmor.mode != SC_AA_NOT_APPLICABLE && getuid() != 0 && geteuid() == 0) {
-        // Refuse to run when this process is running unconfined on a system
-        // that supports AppArmor when the effective uid is root and the real
-        // id is non-root.  This protects against, for example, unprivileged
-        // users trying to leverage the snap-confine in the core snap to
-        // escalate privileges.
-        errno = 0;  // errno is insignificant here
-        die("snap-confine has elevated permissions and is not confined"
-            " but should be. Refusing to continue to avoid"
-            " permission escalation attacks\n"
-            "Please make sure that the snapd.apparmor service is enabled and started.");
-    }
-
     log_startup_stage("snap-confine mount namespace start");
 
     /* perform global initialization of mount namespace support for non-classic
@@ -422,16 +505,16 @@ int main(int argc, char **argv) {
 
     log_startup_stage("snap-confine mount namespace finish");
 
-    // Temporarily drop privileges back to the calling user until we can
-    // permanently drop (which we can't do just yet due to seccomp, see
-    // below).
-    sc_identity real_user_identity = {
-        .uid = real_uid,
-        .gid = real_gid,
-        .change_uid = 1,
-        .change_gid = 1,
-    };
-    sc_set_effective_identity(real_user_identity);
+    // Temporarily drop all effective capabilities, since we don't need any for
+    // a while. Note, we keep CAP_SYS_ADMIN in permitted as it will be needed
+    // later.
+    debug("dropping caps");
+    if (cap_set_proc(caps_no_effective) != 0) {
+        die("cannot drop capabilities");
+    }
+
+    sc_debug_capabilities("after dropping effective caps");
+
     // Ensure that the user data path exists. When creating it use the identity
     // of the calling user (by using real user and group identifiers). This
     // allows the creation of directories inside ~/ on NFS with root_squash
@@ -451,71 +534,79 @@ int main(int argc, char **argv) {
         // for compatibility, if facing older snapd.
         setenv("SNAP_CONTEXT", snap_context, 1);
     }
-    // Normally setuid/setgid not only permanently drops the UID/GID, but
-    // also clears the capabilities bounding sets (see "Effect of user ID
-    // changes on capabilities" in 'man capabilities'). To load a seccomp
-    // profile, we need either CAP_SYS_ADMIN or PR_SET_NO_NEW_PRIVS. Since
-    // NNP causes issues with AppArmor and exec transitions in certain
-    // snapd interfaces, keep CAP_SYS_ADMIN temporarily when we are
-    // permanently dropping privileges.
-    if (getresuid(&real_uid, &effective_uid, &saved_uid) != 0) {
-        die("getresuid failed");
-    }
-    debug("ruid: %d, euid: %d, suid: %d", real_uid, effective_uid, saved_uid);
-    struct __user_cap_header_struct hdr = {_LINUX_CAPABILITY_VERSION_3, 0};
-    struct __user_cap_data_struct cap_data[2] = {{0}};
+    // To load a seccomp profile, we need either CAP_SYS_ADMIN or
+    // PR_SET_NO_NEW_PRIVS. Since NNP causes issues with AppArmor and exec
+    // transitions in certain snapd interfaces, keep CAP_SYS_ADMIN temporarily
+    // when we are permanently dropping privileges.
+    debug("setting capabilities bounding set");
 
-    // At this point in time, if we are going to permanently drop our
-    // effective_uid will not be '0' but our saved_uid will be '0'. Detect
-    // and save when we are in the this state so know when to setup the
-    // capabilities bounding set, regain CAP_SYS_ADMIN and later drop it.
-    bool keep_sys_admin = effective_uid != 0 && saved_uid == 0;
-    if (keep_sys_admin) {
-        debug("setting capabilities bounding set");
-        // clear all 32 bit caps but SYS_ADMIN, with none inheritable
-        cap_data[0].effective = CAP_TO_MASK(CAP_SYS_ADMIN);
-        cap_data[0].permitted = cap_data[0].effective;
-        cap_data[0].inheritable = 0;
-        // clear all 64 bit caps
-        cap_data[1].effective = 0;
-        cap_data[1].permitted = 0;
-        cap_data[1].inheritable = 0;
-        if (capset(&hdr, cap_data) != 0) {
-            die("capset failed");
-        }
-    }
-    // Permanently drop if not root
-    if (effective_uid == 0) {
-        // Note that we do not call setgroups() here because its ok
-        // that the user keeps the groups he already belongs to
-        if (setgid(real_gid) != 0) die("setgid failed");
-        if (setuid(real_uid) != 0) die("setuid failed");
+    /* we're going to drop caps when user isn't root, which preserves the spirit
+     * behind original behavior predating the introduction of non-seuid support
+     * in snap-confine */
+    bool is_regular_user = real_uid != 0;
 
-        if (real_gid != 0 && (getuid() == 0 || geteuid() == 0)) die("permanently dropping privs did not work");
-        if (real_uid != 0 && (getgid() == 0 || getegid() == 0)) die("permanently dropping privs did not work");
+    debug("drop caps as non-root? %s", is_regular_user ? "yes" : "no");
+
+    /* only SYS_ADMIN in effective, keep permitted set unchanged */
+    cap_t cap_only_sys_admin SC_CLEANUP(sc_cleanup_cap_t) = cap_dup(caps_no_effective);
+    static const cap_value_t only_sys_admin_caps[] = {
+        CAP_SYS_ADMIN, /* seccomp */
+    };
+
+    if (cap_set_flag(cap_only_sys_admin, CAP_EFFECTIVE, SC_ARRAY_SIZE(only_sys_admin_caps), only_sys_admin_caps,
+                     CAP_SET) != 0) {
+        die("cannot set capability flags");
     }
-    // Now that we've permanently dropped, regain SYS_ADMIN
-    if (keep_sys_admin) {
-        debug("regaining SYS_ADMIN");
-        cap_data[0].effective = CAP_TO_MASK(CAP_SYS_ADMIN);
-        cap_data[0].permitted = cap_data[0].effective;
-        if (capset(&hdr, cap_data) != 0) {
-            die("capset regain failed");
-        }
+
+    if (cap_set_proc(cap_only_sys_admin) != 0) {
+        die("cannot change capabilities");
     }
+
+    sc_debug_capabilities("before seccomp");
+
     // Now that we've dropped and regained SYS_ADMIN, we can load the
     // seccomp profiles.
     sc_apply_seccomp_profile_for_security_tag(invocation.security_tag);
-    // Even though we set inheritable to 0, let's clear SYS_ADMIN
-    // explicitly
-    if (keep_sys_admin) {
-        debug("clearing SYS_ADMIN");
-        cap_data[0].effective = 0;
-        cap_data[0].permitted = cap_data[0].effective;
-        if (capset(&hdr, cap_data) != 0) {
-            die("capset clear failed");
+
+    if (is_regular_user) {
+        debug("dropping all capabilities for user");
+
+        /* drop all permissions we had as a regular user */
+        cap_t cap_dropped SC_CLEANUP(sc_cleanup_cap_t) = cap_init();
+        if (cap_dropped == NULL) {
+            die("cannot allocate capabilities");
+        }
+        if (cap_set_proc(cap_dropped) != 0) {
+            die("cannot drop capabilities");
+        }
+    } else {
+        debug("restore subset of capabilities for root");
+        /* root, since we're restoring process state we want to keep
+         * DAC_OVERRIDE, such that the some semantics of being root, such as
+         * being able to access non root files (at least while still executing
+         * under snap-confine's profile) are preserved */
+        /* for real root, keep the permitted set, we'll need it later on */
+
+        static const cap_value_t only_dac_override_caps[] = {
+            CAP_DAC_OVERRIDE,
+        };
+        cap_t cap_only_dac_override SC_CLEANUP(sc_cleanup_cap_t) = cap_init();
+        if (cap_only_dac_override == NULL) {
+            die("cannot allocate only DAC caps for root");
+        }
+        if (cap_set_flag(cap_only_dac_override, CAP_EFFECTIVE, SC_ARRAY_SIZE(only_dac_override_caps),
+                         only_dac_override_caps, CAP_SET) != 0 ||
+            cap_set_flag(cap_only_dac_override, CAP_PERMITTED, SC_ARRAY_SIZE(only_dac_override_caps),
+                         only_dac_override_caps, CAP_SET) != 0) {
+            die("cannot set capapbility flags");
+        }
+        if (cap_set_proc(cap_only_dac_override) != 0) {
+            die("cannot drop capabilities");
         }
     }
+
+    sc_debug_capabilities("before exec to application");
+
     // and exec the new executable
     argv[0] = (char *)invocation.executable;
     debug("execv(%s, %s...)", invocation.executable, argv[0]);
@@ -525,6 +616,7 @@ int main(int argc, char **argv) {
     // Restore process state that was recorded earlier.
     sc_restore_process_state(&proc_state);
     log_startup_stage("snap-confine to snap-exec");
+    /* post exec capabilities restored as described in capabilities(7) */
     execv(invocation.executable, (char *const *)&argv[0]);
     perror("execv failed");
     return 1;
@@ -742,6 +834,7 @@ static void enter_non_classic_execution_environment(sc_invocation *inv, struct s
        join. We need to construct a new mount namespace ourselves.
        To capture it we will need a helper process so make one. */
     sc_fork_helper(group, aa);
+    sc_debug_capabilities("caps on join");
     int retval = sc_join_preserved_ns(group, aa, inv, snap_discard_ns_fd);
     if (retval == ESRCH) {
         /* Create and populate the mount namespace. This performs all
@@ -813,7 +906,7 @@ static void enter_non_classic_execution_environment(sc_invocation *inv, struct s
     // assume it makes sense for the core snap layout. Note that the /usr/local
     // directories are explicitly left out as they are not part of the core
     // snap.
-    debug("resetting PATH to values in sync with core snap");
+    debug("resetting PATH and SHELL to values in sync with core snap");
     setenv("PATH",
            "/usr/local/sbin:"
            "/usr/local/bin:"
@@ -824,6 +917,12 @@ static void enter_non_classic_execution_environment(sc_invocation *inv, struct s
            "/usr/games:"
            "/usr/local/games",
            1);
+
+    // Reset SHELL for non-classic snaps to /bin/bash which is present in core,
+    // core18, core20, core22 and core24. The bare base snap does not have a shell
+    // so no special case is necessary there.
+    setenv("SHELL", "/bin/bash", 1);
+
     // Ensure we set the various TMPDIRs to /tmp. One of the parts of setting
     // up the mount namespace is to create a private /tmp directory (this is
     // done in sc_populate_mount_ns() above). The host environment may point to
