@@ -25,6 +25,7 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"log/syslog"
 	"net"
 	"os"
 	"os/exec"
@@ -35,7 +36,7 @@ import (
 	"syscall"
 	"time"
 
-	"github.com/godbus/dbus"
+	"github.com/godbus/dbus/v5"
 	"github.com/jessevdk/go-flags"
 
 	"github.com/snapcore/snapd/client"
@@ -54,6 +55,7 @@ import (
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/snapenv"
 	"github.com/snapcore/snapd/strutil/shlex"
+	"github.com/snapcore/snapd/systemd"
 	"github.com/snapcore/snapd/timeutil"
 	"github.com/snapcore/snapd/x11"
 
@@ -72,7 +74,7 @@ var (
 )
 
 type cmdRun struct {
-	clientMixin
+	mustWaitMixin
 	Command  string `long:"command" hidden:"yes"`
 	HookName string `long:"hook" hidden:"yes"`
 	Revision string `short:"r" default:"unset" hidden:"yes"`
@@ -103,7 +105,12 @@ The run command executes the given snap command with the right confinement
 and environment.
 `),
 		func() flags.Commander {
-			return &cmdRun{}
+			return &cmdRun{
+				mustWaitMixin: mustWaitMixin{
+					noProgress: true,
+					skipAbort:  true,
+				},
+			}
 		}, map[string]string{
 			// TRANSLATORS: This should not start with a lowercase letter.
 			"command": i18n.G("Alternative command to run"),
@@ -150,15 +157,28 @@ func isStopping() (bool, error) {
 	return false, err
 }
 
-func maybeWaitForSecurityProfileRegeneration(cli *client.Client) error {
+const defaultSystemKeyRetryCount = 12
+
+var getSystemKeyRetryCount = func() int {
+	if timeoutEnv := os.Getenv("SNAPD_DEBUG_SYSTEM_KEY_RETRY"); timeoutEnv != "" {
+		if i, err := strconv.Atoi(timeoutEnv); err == nil {
+			return i
+		}
+	}
+
+	return defaultSystemKeyRetryCount
+}
+
+func maybeCheckSystemKeyMismatch(cli *client.Client) (changeID string, err error) {
 	extraData := interfaces.SystemKeyExtraData{
 		AppArmorPrompting: features.AppArmorPrompting.IsEnabled(),
 	}
+
 	// check if the security profiles key has changed, if so, we need
 	// to wait for snapd to re-generate all profiles
-	mismatch, err := interfaces.SystemKeyMismatch(extraData)
+	mismatch, my, err := interfaces.SystemKeyMismatch(extraData)
 	if err == nil && !mismatch {
-		return nil
+		return "", nil
 	}
 	// something went wrong with the system-key compare, try to
 	// reach snapd before continuing
@@ -175,7 +195,7 @@ func maybeWaitForSecurityProfileRegeneration(cli *client.Client) error {
 	}
 	if stopping {
 		logger.Debugf("ignoring system key mismatch during system shutdown/reboot")
-		return nil
+		return "", nil
 	}
 
 	// We have a mismatch, try to connect to snapd, once we can
@@ -190,30 +210,75 @@ func maybeWaitForSecurityProfileRegeneration(cli *client.Client) error {
 	// is to fix the packaging so that snapd is stopped, upgraded
 	// and started.
 	//
-	// connect timeout for client is 5s on each try, so 12*5s = 60s
-	timeout := 12
-	if timeoutEnv := os.Getenv("SNAPD_DEBUG_SYSTEM_KEY_RETRY"); timeoutEnv != "" {
-		if i, err := strconv.Atoi(timeoutEnv); err == nil {
-			timeout = i
-		}
-	}
+	// connect timeout for client is 5s on each try, default retry count is 12,
+	// gives 60s of timeout
+	retryCount := getSystemKeyRetryCount()
 
 	logger.Debugf("system key mismatch detected, waiting for snapd to start responding...")
+	logger.Debugf("my system key: %v", my)
 
-	for i := 0; i < timeout; i++ {
-		// TODO: we could also check cli.Maintenance() here too in case snapd is
-		// down semi-permanently for a refresh, but what message do we show to
-		// the user or what do we do if we know snapd is down for maintenance?
-		if _, err := cli.SysInfo(); err == nil {
-			return nil
-		} else {
-			logger.Debugf("cannot obtain system info: %v", err)
+	for i := 0; i < retryCount; i++ {
+		// refreshes the state of snapd maintenance internally
+		act, err := cli.SystemKeyMismatchAdvice(my)
+		if err == nil {
+			// we have an explicit response from snapd
+			switch act.SuggestedAction {
+			case client.MismatchActionProceed:
+				// we're good to go
+				logger.Debugf("continue execution")
+				return "", nil
+			case client.MismatchActionWaitForChange:
+				// snapd sent us an ID of a change to wait for
+				logger.Debugf("snapd started a regenerate profiles change with ID: %v", act.ChangeID)
+				return act.ChangeID, nil
+			default:
+				return "", fmt.Errorf("internal error: unexpected advice: %q", act.SuggestedAction)
+			}
 		}
+
+		// an error, which could be anything from socket being down, to
+		// relevant method, or our system-key version not being supported by the
+		// API if we're talking to an older snapd, or snapd is simply updating
+		restarting := false
+		if maintErr, ok := cli.Maintenance().(*client.Error); ok && maintErr.Kind == client.ErrorKindDaemonRestart {
+			restarting = true
+		}
+
+		var rspError *client.Error
+		if !restarting && errors.Is(err, client.ErrMismatchAdviceUnsupported) {
+			// snapd does not support system-key mismatch resolution, but it is
+			// responding, so we either woke it up or it was already running. In
+			// the context of home directories on NFS-like mounts, if we woke up
+			// snapd, the profiles would have regenerated and thus would account
+			// for our network mounted home. If snapd was already running when
+			// we poked it, and the security profiles may or may not be up to
+			// date, but since we're misisng an API there is no way for us to
+			// tell snapd to do the update
+			logger.Debugf("system-key mismatch advice not supported by snapd")
+			return "", nil
+		} else if errors.As(err, &rspError) {
+			// actual response error
+			if !restarting || rspError.Kind != client.ErrorKindSystemKeyVersionUnsupported {
+				// we may reach here if snapd is still seeding, in which case we
+				// let the app execution continue, but the app may not function
+				// properly if the sandbox hasn't been set up completely
+				logger.Debugf("ignoring system-key mismatch error: %v", err)
+				return "", nil
+			}
+
+			// edge case when the system-key version we sent is not
+			// supported by snapd, but the daemon appears to be restarting,
+			// hopefully as part of a refresh, and so it makes sense to try again
+			logger.Debugf("system-key version unsupported by snapd, but daemon is restarting")
+		} else {
+			logger.Debugf("cannot obtain system-key mismatch action: %v", err)
+		}
+		// TODO: use ticker
 		// sleep a little bit for good measure
-		time.Sleep(1 * time.Second)
+		<-timeAfter(1 * time.Second)
 	}
 
-	return fmt.Errorf("timeout waiting for snap system profiles to get updated")
+	return "", fmt.Errorf("timeout waiting for snap system profiles to get updated")
 }
 
 func (x *cmdRun) Usage() string {
@@ -248,10 +313,18 @@ func (x *cmdRun) Execute(args []string) error {
 
 	logger.StartupStageTimestamp("start")
 
-	if err := maybeWaitForSecurityProfileRegeneration(x.client); err != nil {
+	regenProfilesChangeID, err := maybeCheckSystemKeyMismatch(x.client)
+	if err != nil {
 		return err
 	}
 
+	if regenProfilesChangeID != "" {
+		logger.Debugf("waiting for security profiles regeneration change: %v", regenProfilesChangeID)
+		if _, err := x.wait(regenProfilesChangeID); err != nil {
+			logger.Debugf("profile regeneration change failed: %v", err)
+		}
+	}
+
 	// Now actually handle the dispatching
 	if x.HookName != "" {
 		return x.snapRunHook(snapApp)
@@ -692,6 +765,31 @@ func snapdHelperPath(toolName string) (string, error) {
 	return filepath.Join(snapBase, dirs.CoreLibExecDir, toolName), nil
 }
 
+/* Kerberos tickets live in /tmp, or in the place specified by KRB5CCNAME
+ * environment variable. However, in snaps /tmp is private.
+ * So rewire the environment variable to point to /var/lib/snapd/hostfs/tmp,
+ * in case the variable is unset or resolves tickets to /tmp/krb5cc*.
+ *
+ * Snaps that want to read Kerberos tickets must then connect to the
+ * kerberos-tickets interface to have access to the tickets.
+ */
+func exposeKerberosTickets(info *snap.Info) (string, error) {
+	krb5EnvVar := osGetenv("KRB5CCNAME")
+	if len(krb5EnvVar) == 0 {
+		return "", nil
+	}
+	if strings.HasPrefix(krb5EnvVar, "FILE:") {
+		path := strings.TrimPrefix(krb5EnvVar, "FILE:")
+		path = filepath.Clean(path)
+		if filepath.Dir(path) == "/tmp" && strings.HasPrefix(path, "/tmp/krb5cc") {
+			// with vanilla config this ends up returning:
+			// FILE:/var/lib/snapd/hostfs/tmp/krb5cc_test
+			return "FILE:" + filepath.Join("/var/lib/snapd/hostfs/", path), nil
+		}
+	}
+	return "", fmt.Errorf("Unsupported KRB5CCNAME: %s", krb5EnvVar)
+}
+
 func migrateXauthority(info *snap.Info) (string, error) {
 	u, err := userCurrent()
 	if err != nil {
@@ -1247,6 +1345,30 @@ func (r *runnable) Validate() error {
 	return nil
 }
 
+func makeStdStreamsForJournal(app *snap.AppInfo, namespace string) (stdout, stderr *os.File) {
+	stdout, err := systemd.NewJournalStreamFile(systemd.JournalStreamFileParams{
+		Namespace:   namespace,
+		Identifier:  app.Name,
+		UnitName:    app.ServiceName(),
+		Priority:    syslog.LOG_DAEMON | syslog.LOG_INFO,
+		LevelPrefix: true,
+	})
+	if err != nil {
+		logger.Noticef("cannot connect to journal for stdout: %s", err)
+	}
+	stderr, err = systemd.NewJournalStreamFile(systemd.JournalStreamFileParams{
+		Namespace:   namespace,
+		Identifier:  app.Name,
+		UnitName:    app.ServiceName(),
+		Priority:    syslog.LOG_DAEMON | syslog.LOG_WARNING,
+		LevelPrefix: true,
+	})
+	if err != nil {
+		logger.Noticef("cannot connect to journal for stderr: %s", err)
+	}
+	return stdout, stderr
+}
+
 func (x *cmdRun) runSnapConfine(info *snap.Info, runner runnable, beforeExec func() error, args []string) error {
 	// check for programmer error, should never happen
 	if err := runner.Validate(); err != nil {
@@ -1281,6 +1403,11 @@ func (x *cmdRun) runSnapConfine(info *snap.Info, runner runnable, beforeExec fun
 		logger.Noticef("WARNING: cannot copy user Xauthority file: %s", err)
 	}
 
+	krb5ccnamePath, err := exposeKerberosTickets(info)
+	if err != nil {
+		logger.Noticef("WARNING: will not expose Kerberos tickets' path: %s", err)
+	}
+
 	if err := activateXdgDocumentPortal(runner); err != nil {
 		logger.Noticef("WARNING: cannot start document portal: %s", err)
 	}
@@ -1365,6 +1492,10 @@ func (x *cmdRun) runSnapConfine(info *snap.Info, runner runnable, beforeExec fun
 		env["XAUTHORITY"] = xauthPath
 	}
 
+	if len(krb5ccnamePath) > 0 {
+		env["KRB5CCNAME"] = krb5ccnamePath
+	}
+
 	// on each run variant path this will be used once to get
 	// the environment plus additions in the right form
 	envForExec := func(extra map[string]string) []string {
@@ -1441,6 +1572,29 @@ func (x *cmdRun) runSnapConfine(info *snap.Info, runner runnable, beforeExec fun
 				return err
 			}
 		}
+
+		// If a journal namespace is supplied for the service, then we reopen stdout/stderr
+		// connected to that journal namespace instead of the main journal. Since we are not
+		// using systemd's LogNamespace= directly, we must do this ourselves.
+		if lns := os.Getenv("SNAPD_LOG_NAMESPACE"); lns != "" {
+			stdout, stderr := makeStdStreamsForJournal(app, lns)
+			if stdout != nil {
+				defer stdout.Close()
+				if err := osutil.DupFD(stdout.Fd(), uintptr(syscall.Stdout)); err != nil {
+					logger.Noticef("cannot duplicate stdout for connection: %v", err)
+				}
+			}
+			if stderr != nil {
+				defer stderr.Close()
+				if err := osutil.DupFD(stderr.Fd(), uintptr(syscall.Stderr)); err != nil {
+					logger.Noticef("cannot duplicate stderr for connection: %v", err)
+				}
+			}
+
+			// Clear out the LOG_NAMESPACE variable, no reason to leak this to the process
+			// itself.
+			os.Unsetenv("SNAPD_LOG_NAMESPACE")
+		}
 	}
 	// Allow using the session bus for all apps but not for hooks.
 	allowSessionBus := !runner.IsHook()
