@@ -21,6 +21,7 @@ package main
 
 import (
 	"bufio"
+	"errors"
 	"fmt"
 	"os"
 	"strconv"
@@ -439,6 +440,15 @@ func (s appName) Complete(match string) []flags.Completion {
 
 type serviceName string
 
+func serviceNames(services []serviceName) []string {
+	names := make([]string, len(services))
+	for i, name := range services {
+		names[i] = string(name)
+	}
+
+	return names
+}
+
 func (s serviceName) Complete(match string) []flags.Completion {
 	cli := mkClient()
 	apps, err := cli.Apps(nil, client.AppOptions{Service: true})
@@ -515,7 +525,7 @@ func (snapshotID) Complete(match string) []flags.Completion {
 func (s snapshotID) ToUint() (uint64, error) {
 	setID, err := strconv.ParseUint((string)(s), 10, 64)
 	if err != nil {
-		return 0, fmt.Errorf(i18n.G("invalid argument for snapshot set id: expected a non-negative integer argument (see 'snap help saved')"))
+		return 0, errors.New(i18n.G("invalid argument for snapshot set id: expected a non-negative integer argument (see 'snap help saved')"))
 	}
 	return setID, nil
 }
