@@ -37,6 +37,7 @@ import (
 	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/overlord/configstate/config"
 	"github.com/snapcore/snapd/overlord/devicestate"
+	"github.com/snapcore/snapd/overlord/ifacestate"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/release"
@@ -53,15 +54,19 @@ var (
 	}
 
 	sysInfoCmd = &Command{
-		Path:       "/v2/system-info",
-		GET:        sysInfo,
-		ReadAccess: interfaceOpenAccess{Interfaces: []string{"snap-interfaces-requests-control"}},
+		Path:        "/v2/system-info",
+		GET:         sysInfo,
+		POST:        sysInfoPost,
+		Actions:     []string{"advise-system-key-mismatch"},
+		ReadAccess:  interfaceOpenAccess{Interfaces: []string{"snap-interfaces-requests-control"}},
+		WriteAccess: openAccess{},
 	}
 
 	stateChangeCmd = &Command{
 		Path:        "/v2/changes/{id}",
 		GET:         getChange,
 		POST:        abortChange,
+		Actions:     []string{"abort"},
 		ReadAccess:  interfaceOpenAccess{Interfaces: []string{"snap-refresh-observe"}},
 		WriteAccess: authenticatedAccess{Polkit: polkitActionManage},
 	}
@@ -76,6 +81,7 @@ var (
 		Path:        "/v2/warnings",
 		GET:         getWarnings,
 		POST:        ackWarnings,
+		Actions:     []string{"okay"},
 		ReadAccess:  openAccess{},
 		WriteAccess: authenticatedAccess{Polkit: polkitActionManage},
 	}
@@ -132,7 +138,7 @@ func sysInfo(c *Command, r *http.Request, user *auth.UserState) Response {
 		refreshInfo.Schedule = refreshScheduleStr
 	}
 
-	m := map[string]interface{}{
+	m := map[string]any{
 		"series":         release.Series,
 		"version":        c.d.Version,
 		"build-id":       buildID,
@@ -140,7 +146,7 @@ func sysInfo(c *Command, r *http.Request, user *auth.UserState) Response {
 		"on-classic":     release.OnClassic,
 		"managed":        len(users) > 0,
 		"kernel-version": osutil.KernelVersion(),
-		"locations": map[string]interface{}{
+		"locations": map[string]any{
 			"snap-mount-dir": dirs.SnapMountDir,
 			"snap-bin-dir":   dirs.SnapBinariesDir,
 		},
@@ -172,6 +178,56 @@ func sysInfo(c *Command, r *http.Request, user *auth.UserState) Response {
 	return SyncResponse(m)
 }
 
+func sysInfoPost(c *Command, r *http.Request, user *auth.UserState) Response {
+	var d struct {
+		Action    string `json:"action"`
+		SystemKey string `json:"system-key"`
+	}
+
+	dec := json.NewDecoder(r.Body)
+	if err := dec.Decode(&d); err != nil {
+		return BadRequest("cannot decode request body: %v", err)
+	}
+
+	if dec.More() {
+		return BadRequest("unexpected additional content in request body")
+	}
+
+	switch d.Action {
+	case "advise-system-key-mismatch":
+	case "":
+		return BadRequest("no action")
+	default:
+		return BadRequest("unsupported action %q", d.Action)
+	}
+
+	var sk any
+	if maybeSk, err := interfaces.SystemKeyFromString(d.SystemKey); err != nil {
+		return BadRequest("cannot decode system key: %v", err)
+	} else {
+		sk = maybeSk
+	}
+
+	st := c.d.state
+	st.Lock()
+	defer st.Unlock()
+
+	logger.Debugf("client reports mismatch with system-key: %v", d.SystemKey)
+	chg, err := ifacestate.AdviseReportedSystemKeyMismatch(c.d.state, sk)
+	if err != nil {
+		return errToResponse(err, nil, InternalError, "cannot process system key: %v")
+	}
+
+	if chg == nil {
+		// proceed
+		return SyncResponse("")
+	}
+
+	ensureStateSoon(c.d.state)
+	// we have a new change
+	return AsyncResponse(nil, chg.ID())
+}
+
 func formatRefreshTime(t time.Time) string {
 	if t.IsZero() {
 		return ""
@@ -321,7 +377,7 @@ type changeInfo struct {
 	Ready   bool        `json:"ready"`
 	Err     string      `json:"err,omitempty"`
 
-	SpawnTime time.Time  `json:"spawn-time,omitempty"`
+	SpawnTime time.Time  `json:"spawn-time,omitzero"`
 	ReadyTime *time.Time `json:"ready-time,omitempty"`
 
 	Data map[string]*json.RawMessage `json:"data,omitempty"`
@@ -335,7 +391,7 @@ type taskInfo struct {
 	Log      []string         `json:"log,omitempty"`
 	Progress taskInfoProgress `json:"progress"`
 
-	SpawnTime time.Time  `json:"spawn-time,omitempty"`
+	SpawnTime time.Time  `json:"spawn-time,omitzero"`
 	ReadyTime *time.Time `json:"ready-time,omitempty"`
 
 	Data map[string]*json.RawMessage `json:"data,omitempty"`
@@ -450,8 +506,6 @@ func getWarnings(c *Command, r *http.Request, _ *auth.UserState) Response {
 	}
 
 	st := c.d.overlord.State()
-	st.Lock()
-	defer st.Unlock()
 
 	var ws []*state.Warning
 	if all {
