@@ -25,6 +25,7 @@ import (
 	"github.com/snapcore/snapd/overlord/devicestate"
 	"github.com/snapcore/snapd/overlord/install"
 	"github.com/snapcore/snapd/overlord/state"
+	"github.com/snapcore/snapd/secboot/keys"
 	"github.com/snapcore/snapd/testutil"
 )
 
@@ -70,10 +71,12 @@ func MockDevicestateRemoveRecoverySystem(f func(*state.State, string) (*state.Ch
 	return restore
 }
 
-func MockDeviceValidatePassphraseOrPINEntropy(f func(device.AuthMode, string) error) (restore func()) {
-	restore = testutil.Backup(&deviceValidatePassphraseOrPINEntropy)
-	deviceValidatePassphraseOrPINEntropy = f
-	return restore
+func MockDevicestateGeneratePreInstallRecoveryKey(f func(st *state.State, label string) (rkey keys.RecoveryKey, err error)) (restore func()) {
+	return testutil.Mock(&devicestateGeneratePreInstallRecoveryKey, f)
+}
+
+func MockDeviceValidatePassphrase(f func(mode device.AuthMode, passphrase string) (device.AuthQuality, error)) (restore func()) {
+	return testutil.Mock(&deviceValidatePassphrase, f)
 }
 
 func ClearCachedEncryptionSupportInfoForLabel(st *state.State, systemLabel string) {
