@@ -95,7 +95,7 @@ var u2fDevices = []u2fDevice{
 	{
 		Name:             "Thetis Key",
 		VendorIDPattern:  "1ea8",
-		ProductIDPattern: "f025",
+		ProductIDPattern: "f025|f825",
 	},
 	{
 		Name:             "Nitrokey FIDO U2F",
@@ -200,7 +200,12 @@ var u2fDevices = []u2fDevice{
 	{
 		Name:             "OneSpan DIGIPASS FX Series",
 		VendorIDPattern:  "1a44",
-		ProductIDPattern: "1501|1502|1503|1506|1507|1508|1509|150A",
+		ProductIDPattern: "1501|1502|1503|1506|1507|1508|1509|150a|150b",
+	},
+	{
+		Name:             "Arculus AuthentiKey",
+		VendorIDPattern:  "3752",
+		ProductIDPattern: "0001",
 	},
 }
 
