@@ -320,6 +320,7 @@ openat
 pause
 personality
 pipe
+~pipe2 - |O_NOTIFICATION_PIPE
 pipe2
 poll
 ppoll
@@ -613,6 +614,7 @@ writev
 pwrite
 pwrite64
 pwritev
+pwritev2
 `)
 
 // Go's net package attempts to bind early to check whether IPv6 is available or not.
