@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2018-2019 Canonical Ltd
+ * Copyright (C) 2018-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -78,6 +78,13 @@ func (dc *groundDeviceContext) SystemMode() string {
 	return dc.systemMode
 }
 
+// Classic returns true if the model is marked as a classic model.
+//
+// TODO: consider refactoring this, since this will return true in cases where
+// we might not really consider the system to be a classic system. For example,
+// this will return true when we're booting into recovery mode on a hybrid
+// system. While the model is a classic model, nothing about the running system
+// looks like a classic system.
 func (dc groundDeviceContext) Classic() bool {
 	return dc.model.Classic()
 }
@@ -99,11 +106,22 @@ func (dc groundDeviceContext) RunMode() bool {
 }
 
 // HasModeenv is true if the grade is set
-// TODO:UC20: will classic devices with uc20 models have a modeenv? I think so?
 func (dc groundDeviceContext) HasModeenv() bool {
 	return dc.model.Grade() != asserts.ModelGradeUnset
 }
 
+// IsCoreBoot is true if the model sports a kernel.
+func (d *groundDeviceContext) IsCoreBoot() bool {
+	return d.model.Kernel() != ""
+}
+
+// IsClassicBoot is true for classic systems with classic initramfs
+// (there are no system modes in this case). If true, the kernel comes
+// from debian packages.
+func (d *groundDeviceContext) IsClassicBoot() bool {
+	return !d.IsCoreBoot()
+}
+
 // expected interface is implemented
 var _ snapstate.DeviceContext = &groundDeviceContext{}
 
