@@ -52,6 +52,7 @@ import (
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/overlord/storecontext"
+	"github.com/snapcore/snapd/overlord/swfeats"
 	"github.com/snapcore/snapd/progress"
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/secboot"
@@ -74,10 +75,49 @@ var (
 	secbootMarkSuccessful = secboot.MarkSuccessful
 )
 
+var (
+	becomeOperationalChangeKind = swfeats.RegisterChangeKind("become-operational")
+	seedChangeKind              = swfeats.RegisterChangeKind("seed")
+	installSystemChangeKind     = swfeats.RegisterChangeKind("install-system")
+	factoryResetChangeKind      = swfeats.RegisterChangeKind("factory-reset")
+)
+
+func init() {
+	swfeats.RegisterEnsure("DeviceManager", "ensureOperational")
+	swfeats.RegisterEnsure("DeviceManager", "ensureSeeded")
+	swfeats.RegisterEnsure("DeviceManager", "ensureAutoImportAssertions")
+	swfeats.RegisterEnsure("DeviceManager", "ensureSerialBoundSystemUserAssertionsProcessed")
+	swfeats.RegisterEnsure("DeviceManager", "ensureBootOk")
+	swfeats.RegisterEnsure("DeviceManager", "ensureCloudInitRestricted")
+	swfeats.RegisterEnsure("DeviceManager", "ensureInstalled")
+	swfeats.RegisterEnsure("DeviceManager", "ensureFactoryReset")
+	swfeats.RegisterEnsure("DeviceManager", "ensureSeedInConfig")
+	swfeats.RegisterEnsure("DeviceManager", "ensureSeedInConfig")
+	swfeats.RegisterEnsure("DeviceManager", "ensureTriedRecoverySystem")
+	swfeats.RegisterEnsure("DeviceManager", "ensurePostFactoryReset")
+	swfeats.RegisterEnsure("DeviceManager", "ensureExpiredUsersRemoved")
+}
+
 // EarlyConfig is a hook set by configstate that can process early configuration
 // during managers' startup.
 var EarlyConfig func(st *state.State, preloadGadget func() (sysconfig.Device, *gadget.Info, error)) error
 
+// ErrNoDeviceIdentityYet is returned when the device doesn't have a serial assertion.
+// It's a special case of ErrNoState.
+var ErrNoDeviceIdentityYet = &noDeviceIdentityYetError{}
+
+// noDeviceIdentityYetError is returned when the device doesn't have a serial assertion.
+type noDeviceIdentityYetError struct{}
+
+func (e *noDeviceIdentityYetError) Error() string {
+	return "device has no identity yet"
+}
+
+func (e *noDeviceIdentityYetError) Is(err error) bool {
+	_, ok := err.(*noDeviceIdentityYetError)
+	return ok || errors.Is(err, state.ErrNoState)
+}
+
 // DeviceManager is responsible for managing the device identity and device
 // policies.
 type DeviceManager struct {
@@ -102,7 +142,10 @@ type DeviceManager struct {
 
 	seedTimings *timings.Timings
 	// this is used during early phases until seeding is under way
-	earlyDeviceSeed     seed.Seed
+	earlyDeviceSeed seed.Seed
+	// these are details about the chosen seed we will be seeding from,
+	// set and valid only before seeding has happened. Should not be
+	// used by tasks that are not explicitly happening prior to system being marked seeded.
 	seedLabel, seedMode string
 	seedChosen          bool
 
@@ -182,6 +225,8 @@ func Manager(s *state.State, hookManager *hookstate.HookManager, runner *state.T
 
 	runner.AddHandler("generate-device-key", m.doGenerateDeviceKey, nil)
 	runner.AddHandler("request-serial", m.doRequestSerial, nil)
+	// Mark-preseeded touches and records the system-key, ensure that it does
+	// not run in parallel with other tasks touching the system-key
 	runner.AddHandler("mark-preseeded", m.doMarkPreseeded, nil)
 	runner.AddHandler("mark-seeded", m.doMarkSeeded, nil)
 	runner.AddHandler("setup-ubuntu-save", m.doSetupUbuntuSave, nil)
@@ -557,6 +602,8 @@ func (m *DeviceManager) ensureOperational() error {
 		return nil
 	}
 
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureOperational")
+
 	perfTimings := timings.New(map[string]string{"ensure": "become-operational"})
 
 	// conditions to trigger device registration
@@ -715,7 +762,7 @@ func (m *DeviceManager) ensureOperational() error {
 		tasks = append(tasks, requestSerial)
 	}
 
-	chg := m.state.NewChange("become-operational", i18n.G("Initialize device"))
+	chg := m.state.NewChange(becomeOperationalChangeKind, i18n.G("Initialize device"))
 	chg.AddAll(state.NewTaskSet(tasks...))
 
 	state.TagTimingsWithChange(perfTimings, chg)
@@ -995,7 +1042,9 @@ func (m *DeviceManager) ensureSeeded() error {
 		return nil
 	}
 
-	chg := m.state.NewChange("seed", "Initialize system state")
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureSeeded")
+
+	chg := m.state.NewChange(seedChangeKind, "Initialize system state")
 	for _, ts := range tsAll {
 		chg.AddAll(ts)
 	}
@@ -1048,6 +1097,8 @@ func (m *DeviceManager) ensureAutoImportAssertions() error {
 		return nil
 	}
 
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureAutoImportAssertions")
+
 	commitTo := func(batch *asserts.Batch) error {
 		return assertstate.AddBatch(m.state, batch, nil)
 	}
@@ -1109,6 +1160,7 @@ func (m *DeviceManager) ensureSerialBoundSystemUserAssertionsProcessed() error {
 		}
 		return err
 	}
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureSerialBoundSystemUserAssertionsProcessed")
 
 	db := assertstate.DB(m.state)
 
@@ -1148,6 +1200,7 @@ func (m *DeviceManager) ensureBootOk() error {
 
 		m.bootOkRan = true
 	}
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureBootOk")
 
 	if !m.bootRevisionsUpdated {
 		if err := snapstate.UpdateBootRevisions(m.state); err != nil {
@@ -1202,6 +1255,7 @@ func (m *DeviceManager) ensureCloudInitRestricted() error {
 	if err != nil {
 		return err
 	}
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureCloudInitRestricted")
 	statusMsg := ""
 
 	switch cloudInitStatus {
@@ -1392,6 +1446,8 @@ func (m *DeviceManager) ensureInstalled() error {
 		return fmt.Errorf("internal error: %v", err)
 	}
 
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureInstalled")
+
 	m.ensureInstalledRan = true
 
 	// Create both setup-run-system and restart-system-to-run-mode tasks as they
@@ -1424,7 +1480,7 @@ func (m *DeviceManager) ensureInstalled() error {
 
 	addTask(restartSystem)
 
-	chg := m.state.NewChange("install-system", i18n.G("Install the system"))
+	chg := m.state.NewChange(installSystemChangeKind, i18n.G("Install the system"))
 	chg.AddAll(state.NewTaskSet(tasks...))
 
 	state.TagTimingsWithChange(perfTimings, chg)
@@ -1457,6 +1513,7 @@ func (m *DeviceManager) ensureFactoryReset() error {
 	if !seeded {
 		return nil
 	}
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureFactoryReset")
 
 	perfTimings := timings.New(map[string]string{"ensure": "factory-reset"})
 
@@ -1502,7 +1559,7 @@ func (m *DeviceManager) ensureFactoryReset() error {
 
 	addTask(restartSystem)
 
-	chg := m.state.NewChange("factory-reset", i18n.G("Perform factory reset"))
+	chg := m.state.NewChange(factoryResetChangeKind, i18n.G("Perform factory reset"))
 	chg.AddAll(state.NewTaskSet(tasks...))
 
 	state.TagTimingsWithChange(perfTimings, chg)
@@ -1575,6 +1632,7 @@ func (m *DeviceManager) ensureSeedInConfig() error {
 			// doMarkSeeded will run "EnsureBefore(0)"
 			return nil
 		}
+		logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureSeedInConfig")
 
 		// Sync seeding with the configuration state. We need to
 		// do this here to ensure that old systems which did not
@@ -1584,6 +1642,8 @@ func (m *DeviceManager) ensureSeedInConfig() error {
 			return err
 		}
 		m.ensureSeedInConfigRan = true
+	} else {
+		logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureSeedInConfig")
 	}
 
 	return nil
@@ -1618,6 +1678,8 @@ func (m *DeviceManager) ensureTriedRecoverySystem() error {
 		return nil
 	}
 
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureTriedRecoverySystem")
+
 	m.state.Lock()
 	defer m.state.Unlock()
 
@@ -1683,6 +1745,8 @@ func (m *DeviceManager) ensurePostFactoryReset() error {
 		return nil
 	}
 
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensurePostFactoryReset")
+
 	m.ensurePostFactoryResetRan = true
 
 	factoryResetMarker := filepath.Join(dirs.SnapDeviceDir, "factory-reset")
@@ -1739,6 +1803,8 @@ func (m *DeviceManager) ensureExpiredUsersRemoved() error {
 		return err
 	}
 
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureExpiredUsersRemoved")
+
 	for _, user := range users {
 		if !user.HasExpired() {
 			continue
@@ -1955,7 +2021,7 @@ func (m *DeviceManager) keyPair() (asserts.PrivateKey, error) {
 }
 
 // SignConfdbControl signs a confdb-control assertion using the device's key as it needs to be attested by the device.
-func (m *DeviceManager) SignConfdbControl(groups []interface{}, revision int) (*asserts.ConfdbControl, error) {
+func (m *DeviceManager) SignConfdbControl(groups []any, revision int) (*asserts.ConfdbControl, error) {
 	serial, err := m.Serial()
 	if err != nil {
 		return nil, fmt.Errorf("cannot sign confdb-control without a serial")
@@ -1966,7 +2032,7 @@ func (m *DeviceManager) SignConfdbControl(groups []interface{}, revision int) (*
 		return nil, fmt.Errorf("cannot sign confdb-control without device key")
 	}
 
-	a, err := asserts.SignWithoutAuthority(asserts.ConfdbControlType, map[string]interface{}{
+	a, err := asserts.SignWithoutAuthority(asserts.ConfdbControlType, map[string]any{
 		"brand-id": serial.BrandID(),
 		"model":    serial.Model(),
 		"serial":   serial.Serial(),
@@ -2055,6 +2121,35 @@ func (m *DeviceManager) Serial() (*asserts.Serial, error) {
 	return findSerial(m.state, nil)
 }
 
+// ConfdbControl returns the device's confdb-control assertion.
+func (m *DeviceManager) ConfdbControl() (*asserts.ConfdbControl, error) {
+	serial, err := m.Serial()
+	if err != nil {
+		return nil, ErrNoDeviceIdentityYet
+	}
+
+	db := assertstate.DB(m.state)
+	a, err := db.Find(asserts.ConfdbControlType, map[string]string{
+		"brand-id": serial.BrandID(),
+		"model":    serial.Model(),
+		"serial":   serial.Serial(),
+	})
+	if errors.Is(err, &asserts.NotFoundError{}) {
+		return nil, state.ErrNoState
+	}
+	if err != nil {
+		return nil, err
+	}
+
+	cc := a.(*asserts.ConfdbControl)
+	key := serial.DeviceKey()
+	if key.ID() != cc.SignKeyID() {
+		return nil, errors.New("confdb-control's signing key doesn't match the device key")
+	}
+
+	return cc, nil
+}
+
 type SystemModeInfo struct {
 	Mode              string
 	HasModeenv        bool
@@ -2604,7 +2699,7 @@ func (scb storeContextBackend) SignDeviceSessionRequest(serial *asserts.Serial,
 		return nil, err
 	}
 
-	a, err := asserts.SignWithoutAuthority(asserts.DeviceSessionRequestType, map[string]interface{}{
+	a, err := asserts.SignWithoutAuthority(asserts.DeviceSessionRequestType, map[string]any{
 		"brand-id":  serial.BrandID(),
 		"model":     serial.Model(),
 		"serial":    serial.Serial(),
@@ -2691,7 +2786,7 @@ func (m *DeviceManager) runFDESetupHook(req *fde.SetupRequest) ([]byte, error) {
 		// XXX: should this be configurable somehow?
 		Timeout: 5 * time.Minute,
 	}
-	contextData := map[string]interface{}{
+	contextData := map[string]any{
 		"fde-setup-request": req,
 	}
 	st.Unlock()
