@@ -21,7 +21,6 @@ package devicestate
 
 import (
 	"context"
-	"fmt"
 	"net/http"
 	"time"
 
@@ -35,6 +34,7 @@ import (
 	"github.com/snapcore/snapd/kernel/fde"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/user"
+	"github.com/snapcore/snapd/overlord/fdestate"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/overlord/storecontext"
@@ -121,6 +121,10 @@ func EnsureOperationalShouldBackoff(m *DeviceManager, now time.Time) bool {
 	return m.ensureOperationalShouldBackoff(now)
 }
 
+func LoadSystemAndEssentialSnaps(m *DeviceManager, wantedSystemLabel string, types []snap.Type, modeForComps string) (*systemAndEssentialSnaps, error) {
+	return m.loadSystemAndEssentialSnaps(wantedSystemLabel, types, modeForComps)
+}
+
 func BecomeOperationalBackoff(m *DeviceManager) time.Duration {
 	return m.becomeOperationalBackoff
 }
@@ -572,7 +576,7 @@ func MockCreateAllKnownSystemUsers(createAllUsers func(state *state.State, asser
 	return restore
 }
 
-func MockEncryptionSetupDataInCache(st *state.State, label string, volumesAuth *device.VolumesAuthOptions) (restore func()) {
+func MockEncryptionSetupDataInCache(st *state.State, label string, recoveryKeyID string, volumesAuth *device.VolumesAuthOptions) (restore func()) {
 	st.Lock()
 	defer st.Unlock()
 	var esd *install.EncryptionSetupData
@@ -586,20 +590,17 @@ func MockEncryptionSetupDataInCache(st *state.State, label string, volumesAuth *
 			EncryptedDevice: "/dev/mapper/ubuntu-data",
 		},
 	}
-	esd = install.MockEncryptionSetupData(labelToEncData, volumesAuth)
+	esd = install.MockEncryptionSetupData(labelToEncData, recoveryKeyID, volumesAuth)
 	st.Cache(encryptionSetupDataKey{label}, esd)
 	return func() { CleanUpEncryptionSetupDataInCache(st, label) }
 }
 
-func CheckEncryptionSetupDataFromCache(st *state.State, label string) error {
+func GetEncryptionSetupDataFromCache(st *state.State, label string) *install.EncryptionSetupData {
 	cached := st.Cached(encryptionSetupDataKey{label})
 	if cached == nil {
-		return fmt.Errorf("no EncryptionSetupData found in cache")
+		return nil
 	}
-	if _, ok := cached.(*install.EncryptionSetupData); !ok {
-		return fmt.Errorf("wrong data type under encryptionSetupDataKey")
-	}
-	return nil
+	return cached.(*install.EncryptionSetupData)
 }
 
 func CleanUpEncryptionSetupDataInCache(st *state.State, label string) {
@@ -619,11 +620,11 @@ func MockSecbootAddBootstrapKeyOnExistingDisk(f func(node string, newKey keys.En
 	}
 }
 
-func MockSecbootRenameKeys(f func(node string, renames map[string]string) error) (restore func()) {
-	old := secbootRenameKeys
-	secbootRenameKeys = f
+func MockSecbootRenameKeysForFactoryReset(f func(node string, renames map[string]string) error) (restore func()) {
+	old := secbootRenameKeysForFactoryReset
+	secbootRenameKeysForFactoryReset = f
 	return func() {
-		secbootRenameKeys = old
+		secbootRenameKeysForFactoryReset = old
 	}
 }
 
@@ -678,3 +679,19 @@ func MockSecbootRemoveOldCounterHandles(f func(node string, possibleOldKeys map[
 		secbootRemoveOldCounterHandles = old
 	}
 }
+
+func MockFdestateGenerateRecoveryKey(f func(st *state.State) (rkey keys.RecoveryKey, keyID string, err error)) (restore func()) {
+	return testutil.Mock(&fdestateGenerateRecoveryKey, f)
+}
+
+func MockFdestateGetRecoveryKey(f func(st *state.State, keyID string) (rkey keys.RecoveryKey, err error)) (restore func()) {
+	return testutil.Mock(&fdestateGetRecoveryKey, f)
+}
+
+func MockFdestateGetKeyslots(f func(st *state.State, keyslotRefs []fdestate.KeyslotRef) (keyslots []fdestate.Keyslot, missingRefs []fdestate.KeyslotRef, err error)) (restore func()) {
+	return testutil.Mock(&fdestateGetKeyslots, f)
+}
+
+func MockSnapstateGadgetInfo(f func(st *state.State, deviceCtx snapstate.DeviceContext) (*snap.Info, error)) (restore func()) {
+	return testutil.Mock(&snapstateGadgetInfo, f)
+}
