@@ -46,6 +46,7 @@ import (
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/disks"
+	"github.com/snapcore/snapd/overlord/fdestate"
 	installLogic "github.com/snapcore/snapd/overlord/install"
 	"github.com/snapcore/snapd/overlord/restart"
 	"github.com/snapcore/snapd/overlord/snapstate"
@@ -79,6 +80,8 @@ var (
 	secbootTransitionEncryptionKeyChange = secboot.TransitionEncryptionKeyChange
 	secbootRemoveOldCounterHandles       = secboot.RemoveOldCounterHandles
 	secbootTemporaryNameOldKeys          = secboot.TemporaryNameOldKeys
+	fdestateGetRecoveryKey               = fdestate.GetRecoveryKey
+	fdestateGenerateRecoveryKey          = fdestate.GenerateRecoveryKey
 
 	installLogicPrepareRunSystemData = installLogic.PrepareRunSystemData
 )
@@ -282,7 +285,7 @@ func (m *DeviceManager) doSetupRunSystem(t *state.Task, _ *tomb.Tomb) error {
 
 	// Load seed to find out kernel-modules components in run mode
 	systemAndSnaps, mntPtForType, mntPtForComps, unmount,
-		err := m.loadAndMountSystemLabelSnapsUnlock(st, m.seedLabel, []snap.Type{snap.TypeKernel})
+		err := m.loadAndMountSystemLabelSnapsUnlock(st, modeEnv.RecoverySystem, []snap.Type{snap.TypeKernel})
 	if err != nil {
 		return err
 	}
@@ -564,7 +567,7 @@ func (m *DeviceManager) doFactoryResetRunSystem(t *state.Task, _ *tomb.Tomb) err
 
 	// Load seed to find out kernel-modules components in run mode
 	systemAndSnaps, mntPtForType, mntPtForComps, unmount,
-		err := m.loadAndMountSystemLabelSnapsUnlock(st, m.seedLabel, []snap.Type{snap.TypeKernel})
+		err := m.loadAndMountSystemLabelSnapsUnlock(st, modeEnv.RecoverySystem, []snap.Type{snap.TypeKernel})
 	if err != nil {
 		return err
 	}
@@ -900,13 +903,6 @@ func (m *DeviceManager) loadAndMountSystemLabelSnaps(systemLabel string, essenti
 	if err != nil {
 		return nil, nil, nil, nil, err
 	}
-	// Unset revision here actually means that the snap is local.
-	// Assign then a local revision as seeding/installing the snap would do.
-	for _, snInfo := range systemAndSnaps.InfosByType {
-		if snInfo.Revision.Unset() {
-			snInfo.Revision = snap.R(-1)
-		}
-	}
 
 	// Mount gadget, kernel and kernel-modules components
 	var unmountFuncs []func() error
@@ -964,6 +960,17 @@ func kBootInfo(systemAndSnaps *systemAndEssentialSnaps, kernMntPoint string, mnt
 			IsCore: isCore, NeedsDriversTree: kernel.NeedsKernelDriversTree(systemAndSnaps.Model)})
 }
 
+func addPreInstallRecoveryKey(dataBootstrappedContainer, saveBootstrappedContainer secboot.BootstrappedContainer, rkey keys.RecoveryKey) error {
+	if dataBootstrappedContainer == nil || saveBootstrappedContainer == nil {
+		return errors.New("internal error: system encryption keys are unset")
+	}
+
+	if err := dataBootstrappedContainer.AddRecoveryKey("default-recovery", rkey); err != nil {
+		return err
+	}
+	return saveBootstrappedContainer.AddRecoveryKey("default-recovery", rkey)
+}
+
 // doInstallFinish performs the finish step of the install. It will
 // - install missing volumes structure content
 // - copy seed (only for UC)
@@ -1118,8 +1125,7 @@ func (m *DeviceManager) doInstallFinish(t *state.Task, _ *tomb.Tomb) error {
 			return fmt.Errorf("cannot copy seed: %w", err)
 		}
 
-		hybrid := systemAndSnaps.Model.Classic() && systemAndSnaps.Model.KernelSnap() != nil
-		if hybrid {
+		if systemAndSnaps.Model.HybridClassic() {
 			// boot.InitramfsUbuntuSeedDir (/run/mnt/ubuntu-data, usually) is a
 			// mountpoint on hybrid system that is set up in the initramfs.
 			// setting up this bind mount ensures that the system can be seeded
@@ -1165,8 +1171,20 @@ func (m *DeviceManager) doInstallFinish(t *state.Task, _ *tomb.Tomb) error {
 	}
 
 	if useEncryption {
+		bootstrappedContainersForRole := install.BootstrappedContainersForRole(encryptSetupData)
 		if trustedInstallObserver != nil {
-			if err := installLogic.PrepareEncryptedSystemData(systemAndSnaps.Model, install.BootstrappedContainersForRole(encryptSetupData), encryptSetupData.VolumesAuth(), trustedInstallObserver); err != nil {
+			if err := installLogic.PrepareEncryptedSystemData(systemAndSnaps.Model, bootstrappedContainersForRole, encryptSetupData.VolumesAuth(), trustedInstallObserver); err != nil {
+				return err
+			}
+		}
+
+		recoveryKeyID := encryptSetupData.RecoveryKeyID()
+		if recoveryKeyID != "" {
+			rkey, err := fdestateGetRecoveryKey(st, recoveryKeyID)
+			if err != nil {
+				return err
+			}
+			if err := addPreInstallRecoveryKey(bootstrappedContainersForRole[gadget.SystemData], bootstrappedContainersForRole[gadget.SystemSave], rkey); err != nil {
 				return err
 			}
 		}
@@ -1329,7 +1347,7 @@ func (m *DeviceManager) doInstallSetupStorageEncryption(t *state.Task, _ *tomb.T
 	}
 
 	// Store created devices in the change so they can be accessed from the installer
-	apiData := map[string]interface{}{
+	apiData := map[string]any{
 		"encrypted-devices": encryptionSetupData.EncryptedDevices(),
 	}
 	chg := t.Change()
@@ -1342,7 +1360,7 @@ func (m *DeviceManager) doInstallSetupStorageEncryption(t *state.Task, _ *tomb.T
 
 var (
 	secbootAddBootstrapKeyOnExistingDisk = secboot.AddBootstrapKeyOnExistingDisk
-	secbootRenameKeys                    = secboot.RenameKeys
+	secbootRenameKeysForFactoryReset     = secboot.RenameKeysForFactoryReset
 	secbootCreateBootstrappedContainer   = secboot.CreateBootstrappedContainer
 	secbootDeleteKeys                    = secboot.DeleteKeys
 	secbootDeleteOldKeys                 = secboot.DeleteOldKeys
@@ -1387,7 +1405,7 @@ func createSaveBootstrappedContainer(saveNode string) (secboot.BootstrappedConta
 	}
 	// Temporarily rename keyslots across the factory reset to
 	// allow to create the new ones.
-	if err := secbootRenameKeys(saveNode, renames); err != nil {
+	if err := secbootRenameKeysForFactoryReset(saveNode, renames); err != nil {
 		return nil, fmt.Errorf("cannot rename existing keys: %w", err)
 	}
 
@@ -1472,3 +1490,34 @@ func rotateSaveKeyAndDeleteOldKeys(saveMntPnt string) error {
 	}
 	return nil
 }
+
+// GeneratePreInstallRecoveryKey generates a recovery key and embeds
+// its corresponding id in the storage encryption setup data.
+//
+// Note: InstallSetupStorageEncryption must be called before calling
+// this helper.
+func GeneratePreInstallRecoveryKey(st *state.State, label string) (rkey keys.RecoveryKey, err error) {
+	var encryptSetupData *install.EncryptionSetupData
+	cached := st.Cached(encryptionSetupDataKey{label})
+	if cached == nil {
+		return keys.RecoveryKey{}, fmt.Errorf("storage encryption setup step was not called")
+	}
+
+	// XXX: just let it panic?
+	encryptSetupData, ok := cached.(*install.EncryptionSetupData)
+	if !ok {
+		return keys.RecoveryKey{}, fmt.Errorf("internal error: wrong data type under encryptionSetupDataKey")
+	}
+
+	rkey, keyID, err := fdestateGenerateRecoveryKey(st)
+	if err != nil {
+		return keys.RecoveryKey{}, err
+	}
+
+	// attach key-id to encryption setup data so it can be used
+	// in the install finish step.
+	encryptSetupData.SetRecoveryKeyID(keyID)
+	st.Cache(encryptionSetupDataKey{label}, encryptSetupData)
+
+	return rkey, err
+}
