@@ -20,10 +20,11 @@
 package backend
 
 import (
+	"github.com/snapcore/snapd/boot"
 	"github.com/snapcore/snapd/secboot"
 )
 
-func MockSecbootResealKeysWithFDESetupHook(f func(keys []secboot.KeyDataLocation, primaryKeyFile string, models []secboot.ModelForSealing, bootModes []string) error) (restore func()) {
+func MockSecbootResealKeysWithFDESetupHook(f func(keys []secboot.KeyDataLocation, primaryKeyGetter func() ([]byte, error), models []secboot.ModelForSealing, bootModes []string) error) (restore func()) {
 	old := secbootResealKeysWithFDESetupHook
 	secbootResealKeysWithFDESetupHook = f
 	return func() {
@@ -38,3 +39,19 @@ func MockSsecbootFindFreeHandle(f func() (uint32, error)) (restore func()) {
 		secbootFindFreeHandle = old
 	}
 }
+
+func MockSecbootRevokeOldKeys(f func(uk *secboot.UpdatedKeys, primaryKey []byte) error) (restore func()) {
+	old := secbootRevokeOldKeys
+	secbootRevokeOldKeys = f
+	return func() {
+		secbootRevokeOldKeys = old
+	}
+}
+
+func MockBootIsResealNeeded(f func(pbc boot.PredictableBootChains, bootChainsFile string, expectReseal bool) (ok bool, nextCount int, err error)) (restore func()) {
+	old := bootIsResealNeeded
+	bootIsResealNeeded = f
+	return func() {
+		bootIsResealNeeded = old
+	}
+}
