@@ -39,9 +39,8 @@ import (
 )
 
 var (
-	confdbstateGetView              = confdbstate.GetView
-	confdbstateNewTransaction       = confdbstate.NewTransaction
-	confdbstateGetStoredTransaction = confdbstate.GetStoredTransaction
+	confdbstateGetView           = confdbstate.GetView
+	confdbstateTransactionForGet = confdbstate.GetTransactionForSnapctlGet
 )
 
 type getCommand struct {
@@ -51,7 +50,7 @@ type getCommand struct {
 	ForceSlotSide bool `long:"slot" description:"return attribute values from the slot side of the connection"`
 	ForcePlugSide bool `long:"plug" description:"return attribute values from the plug side of the connection"`
 	View          bool `long:"view" description:"return confdb values from the view declared in the plug"`
-	Pristine      bool `long:"pristine" description:"return confdb values disregarding changes from the current transaction"`
+	Previous      bool `long:"previous" description:"return confdb values disregarding changes from the current transaction"`
 
 	Positional struct {
 		PlugOrSlotSpec string   `positional-args:"true" positional-arg-name:":<plug|slot>"`
@@ -104,8 +103,8 @@ func init() {
 	})
 }
 
-func (c *getCommand) printValues(getByKey func(string) (interface{}, bool, error)) error {
-	patch := make(map[string]interface{})
+func (c *getCommand) printValues(getByKey func(string) (any, bool, error)) error {
+	patch := make(map[string]any)
 	for _, key := range c.Positional.Keys {
 		value, output, err := getByKey(key)
 		if err == nil {
@@ -120,10 +119,10 @@ func (c *getCommand) printValues(getByKey func(string) (interface{}, bool, error
 	return c.printPatch(patch)
 }
 
-func (c *getCommand) printPatch(patch interface{}) error {
-	var confToPrint interface{} = patch
+func (c *getCommand) printPatch(patch any) error {
+	var confToPrint any = patch
 	if !c.Document && len(c.Positional.Keys) == 1 {
-		if confMap, ok := patch.(map[string]interface{}); ok {
+		if confMap, ok := patch.(map[string]any); ok {
 			confToPrint = confMap[c.Positional.Keys[0]]
 		}
 	}
@@ -165,8 +164,16 @@ func (c *getCommand) Execute(args []string) error {
 	if c.Typed && c.Document {
 		return fmt.Errorf("cannot use -d and -t together")
 	}
-	if c.Pristine && !c.View {
-		return fmt.Errorf("cannot use --pristine without --view")
+	if c.Previous {
+		if !c.View {
+			return fmt.Errorf("cannot use --previous without --view")
+		}
+
+		hookPref := func(p string) bool { return strings.HasPrefix(context.HookName(), p) }
+		if context == nil || context.IsEphemeral() || !(hookPref("save-view-") ||
+			hookPref("change-view-") || hookPref("observe-view-")) {
+			return fmt.Errorf(`cannot use --previous outside of save-view, change-view or observe-view hooks`)
+		}
 	}
 
 	if strings.Contains(c.Positional.PlugOrSlotSpec, ":") {
@@ -185,7 +192,7 @@ func (c *getCommand) Execute(args []string) error {
 			}
 
 			requests := c.Positional.Keys
-			return c.getConfdbValues(context, name, requests, c.Pristine)
+			return c.getConfdbValues(context, name, requests, c.Previous)
 		}
 
 		if len(c.Positional.Keys) == 0 {
@@ -211,8 +218,8 @@ func (c *getCommand) getConfigSetting(context *hookstate.Context) error {
 	transaction := configstate.ContextTransaction(context)
 	context.Unlock()
 
-	return c.printValues(func(key string) (interface{}, bool, error) {
-		var value interface{}
+	return c.printValues(func(key string) (any, bool, error) {
+		var value any
 		err := transaction.Get(c.context().InstanceName(), key, &value)
 		if err == nil {
 			return value, true, nil
@@ -342,7 +349,7 @@ func (c *getCommand) getInterfaceSetting(context *hookstate.Context, plugOrSlot
 	st.Lock()
 	defer st.Unlock()
 
-	var staticAttrs, dynamicAttrs map[string]interface{}
+	var staticAttrs, dynamicAttrs map[string]any
 	if err = attrsTask.Get(which+"-static", &staticAttrs); err != nil {
 		return fmt.Errorf(i18n.G("internal error: cannot get %s from appropriate task"), which)
 	}
@@ -350,13 +357,13 @@ func (c *getCommand) getInterfaceSetting(context *hookstate.Context, plugOrSlot
 		return fmt.Errorf(i18n.G("internal error: cannot get %s from appropriate task"), which)
 	}
 
-	return c.printValues(func(key string) (interface{}, bool, error) {
+	return c.printValues(func(key string) (any, bool, error) {
 		subkeys, err := config.ParseKey(key)
 		if err != nil {
 			return nil, false, err
 		}
 
-		var value interface{}
+		var value any
 		err = getAttribute(context.InstanceName(), subkeys, 0, staticAttrs, &value)
 		if err == nil {
 			return value, true, nil
@@ -371,28 +378,33 @@ func (c *getCommand) getInterfaceSetting(context *hookstate.Context, plugOrSlot
 	})
 }
 
-func (c *getCommand) getConfdbValues(ctx *hookstate.Context, plugName string, requests []string, pristine bool) error {
+func (c *getCommand) getConfdbValues(ctx *hookstate.Context, plugName string, requests []string, previous bool) error {
 	if c.ForcePlugSide || c.ForceSlotSide {
 		return errors.New(i18n.G("cannot use --plug or --slot with --view"))
 	}
 	ctx.Lock()
 	defer ctx.Unlock()
 
-	account, confdbName, viewName, err := getConfdbViewID(ctx, plugName)
+	account, dbSchemaName, viewName, err := getConfdbViewID(ctx, plugName)
 	if err != nil {
 		return err
 	}
 
-	view, err := confdbstateGetView(ctx.State(), account, confdbName, viewName)
+	view, err := confdbstateGetView(ctx.State(), account, dbSchemaName, viewName)
 	if err != nil {
 		return err
 	}
 
-	bag, err := c.getDatabag(ctx, view, pristine)
+	tx, err := confdbstateTransactionForGet(ctx, view, requests)
 	if err != nil {
 		return err
 	}
 
+	var bag confdb.Databag = tx
+	if previous {
+		bag = tx.Previous()
+	}
+
 	res, err := confdbstate.GetViaView(bag, view, requests)
 	if err != nil {
 		return err
@@ -401,41 +413,7 @@ func (c *getCommand) getConfdbValues(ctx *hookstate.Context, plugName string, re
 	return c.printPatch(res)
 }
 
-func (c *getCommand) getDatabag(ctx *hookstate.Context, view *confdb.View, pristine bool) (bag confdb.DataBag, err error) {
-	account, confdbName := view.Confdb().Account, view.Confdb().Name
-
-	var tx *confdbstate.Transaction
-	if confdbstate.IsConfdbHook(ctx) {
-		// running in the context of a transaction, so if the referenced confdb
-		// doesn't match that tx, we only allow the caller to read the other confdb
-		t, _ := ctx.Task()
-		tx, _, err = confdbstateGetStoredTransaction(t)
-		if err != nil {
-			return nil, fmt.Errorf("cannot access confdb view %s/%s/%s: cannot get transaction: %v", account, confdbName, view.Name, err)
-		}
-
-		if tx.ConfdbAccount != account || tx.ConfdbName != confdbName {
-			// we're reading a different transaction
-			tx = nil
-		}
-	}
-
-	// reading a view but there's no ongoing transaction for it, make a temporary
-	// transaction just as a pass-through databag
-	if tx == nil {
-		tx, err = confdbstateNewTransaction(ctx.State(), account, confdbName)
-		if err != nil {
-			return nil, err
-		}
-	}
-
-	if pristine {
-		return tx.Pristine(), nil
-	}
-	return tx, nil
-}
-
-func getConfdbViewID(ctx *hookstate.Context, plugName string) (account, confdbName, viewName string, err error) {
+func getConfdbViewID(ctx *hookstate.Context, plugName string) (account, dbSchemaName, viewName string, err error) {
 	repo := ifacerepo.Get(ctx.State())
 
 	plug := repo.Plug(ctx.InstanceName(), plugName)
@@ -447,29 +425,29 @@ func getConfdbViewID(ctx *hookstate.Context, plugName string) (account, confdbNa
 		return "", "", "", fmt.Errorf(i18n.G("cannot use --view with non-confdb plug :%s"), plugName)
 	}
 
-	account, confdbName, viewName, err = snap.ConfdbPlugAttrs(plug)
+	account, dbSchemaName, viewName, err = snap.ConfdbPlugAttrs(plug)
 	if err != nil {
 		return "", "", "", fmt.Errorf(i18n.G("invalid plug :%s: %w"), plugName, err)
 	}
 
-	return account, confdbName, viewName, nil
+	return account, dbSchemaName, viewName, nil
 }
 
-// validateConfdbsFeatureFlag checks whether the confdbs experimental flag
+// validateConfdbsFeatureFlag checks whether the confdb experimental flag
 // is enabled. The state should not be locked by the caller.
 func validateConfdbsFeatureFlag(st *state.State) error {
 	st.Lock()
 	defer st.Unlock()
 
 	tr := config.NewTransaction(st)
-	enabled, err := features.Flag(tr, features.Confdbs)
+	enabled, err := features.Flag(tr, features.Confdb)
 	if err != nil && !config.IsNoOption(err) {
-		return fmt.Errorf(i18n.G("internal error: cannot check confdbs feature flag: %v"), err)
+		return fmt.Errorf(i18n.G("internal error: cannot check confdb feature flag: %v"), err)
 	}
 
 	if !enabled {
-		_, confName := features.Confdbs.ConfigOption()
-		return fmt.Errorf(i18n.G(`"confdbs" feature flag is disabled: set '%s' to true`), confName)
+		_, confName := features.Confdb.ConfigOption()
+		return fmt.Errorf(i18n.G(`"confdb" feature flag is disabled: set '%s' to true`), confName)
 	}
 
 	return nil
