@@ -44,6 +44,7 @@ import (
 	"github.com/snapcore/snapd/overlord/ifacestate/ifacerepo"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/store"
 	"github.com/snapcore/snapd/testutil"
 )
 
@@ -165,7 +166,7 @@ func (s *getSuite) TestGetTests(c *C) {
 
 var getTests2 = []struct {
 	setPath      string
-	setValue     interface{}
+	setValue     any
 	args, stdout string
 }{{
 	setPath:  "root.key1",
@@ -184,7 +185,7 @@ var getTests2 = []struct {
 	stdout:   "{\n\t\"sub1\": \"z\",\n\t\"sub2\": \"y\"\n}\n",
 }, {
 	setPath:  "root.key3",
-	setValue: map[string]interface{}{"sub3": "z"},
+	setValue: map[string]any{"sub3": "z"},
 	args:     "get root",
 	stdout:   "{\n\t\"key1\": \"a\",\n\t\"key2\": \"b\",\n\t\"key3\": {\n\t\t\"sub3\": \"z\"\n\t}\n}\n",
 }}
@@ -207,7 +208,7 @@ func (s *getSuite) TestGetPartialNestedStruct(c *C) {
 
 		// Initialize configuration
 		tr := config.NewTransaction(state)
-		tr.Set("test-snap", "root", map[string]interface{}{"key1": "a", "key2": "b", "key3": map[string]interface{}{"sub1": "x", "sub2": "y"}})
+		tr.Set("test-snap", "root", map[string]any{"key1": "a", "key2": "b", "key3": map[string]any{"sub1": "x", "sub2": "y"}})
 		tr.Commit()
 
 		state.Unlock()
@@ -229,9 +230,9 @@ func (s *getSuite) TestGetPartialNestedStruct(c *C) {
 		state.Lock()
 		defer state.Unlock()
 		tr3 := config.NewTransaction(state)
-		var config map[string]interface{}
+		var config map[string]any
 		c.Assert(tr3.Get("test-snap", "root", &config), IsNil)
-		c.Assert(config, DeepEquals, map[string]interface{}{"key1": "a", "key2": "b", "key3": map[string]interface{}{"sub1": "x", "sub2": "y"}})
+		c.Assert(config, DeepEquals, map[string]any{"key1": "a", "key2": "b", "key3": map[string]any{"sub1": "x", "sub2": "y"}})
 	}
 }
 
@@ -276,11 +277,11 @@ func (s *setSuite) TestNull(c *C) {
 	c.Check(s.mockContext.Done(), IsNil)
 
 	// Verify config value
-	var value interface{}
+	var value any
 	tr := config.NewTransaction(s.mockContext.State())
 	c.Assert(config.IsNoOption(tr.Get("test-snap", "foo", &value)), Equals, true)
 	c.Assert(tr.Get("test-snap", "bar", &value), IsNil)
-	c.Assert(value, DeepEquals, []interface{}{nil})
+	c.Assert(value, DeepEquals, []any{nil})
 }
 
 func (s *getAttrSuite) SetUpTest(c *C) {
@@ -293,20 +294,20 @@ func (s *getAttrSuite) SetUpTest(c *C) {
 	attrsTask := state.NewTask("connect-task", "my connect task")
 	attrsTask.Set("plug", &interfaces.PlugRef{Snap: "a", Name: "aplug"})
 	attrsTask.Set("slot", &interfaces.SlotRef{Snap: "b", Name: "bslot"})
-	staticPlugAttrs := map[string]interface{}{
+	staticPlugAttrs := map[string]any{
 		"aattr":   "foo",
 		"baz":     []string{"a", "b"},
-		"mapattr": map[string]interface{}{"mapattr1": "mapval1", "mapattr2": "mapval2"},
+		"mapattr": map[string]any{"mapattr1": "mapval1", "mapattr2": "mapval2"},
 	}
-	dynamicPlugAttrs := map[string]interface{}{
+	dynamicPlugAttrs := map[string]any{
 		"dyn-plug-attr": "c",
 		"nilattr":       nil,
 	}
-	dynamicSlotAttrs := map[string]interface{}{
+	dynamicSlotAttrs := map[string]any{
 		"dyn-slot-attr": "d",
 	}
 
-	staticSlotAttrs := map[string]interface{}{
+	staticSlotAttrs := map[string]any{
 		"battr": "bar",
 	}
 	attrsTask.Set("plug-static", staticPlugAttrs)
@@ -507,21 +508,21 @@ func (s *confdbSuite) SetUpTest(c *C) {
 	c.Check(s.signingDB, NotNil)
 	c.Assert(storeSigning.Add(devAccKey), IsNil)
 
-	headers := map[string]interface{}{
+	headers := map[string]any{
 		"authority-id": s.devAccID,
 		"account-id":   s.devAccID,
 		"name":         "network",
-		"views": map[string]interface{}{
-			"read-wifi": map[string]interface{}{
-				"rules": []interface{}{
-					map[string]interface{}{"request": "ssid", "storage": "wifi.ssid", "access": "read"},
-					map[string]interface{}{"request": "password", "storage": "wifi.psk", "access": "read"},
+		"views": map[string]any{
+			"read-wifi": map[string]any{
+				"rules": []any{
+					map[string]any{"request": "ssid", "storage": "wifi.ssid", "access": "read"},
+					map[string]any{"request": "password", "storage": "wifi.psk", "access": "read"},
 				},
 			},
-			"write-wifi": map[string]interface{}{
-				"rules": []interface{}{
-					map[string]interface{}{"request": "ssid", "storage": "wifi.ssid", "access": "write"},
-					map[string]interface{}{"request": "password", "storage": "wifi.psk", "access": "write"},
+			"write-wifi": map[string]any{
+				"rules": []any{
+					map[string]any{"request": "ssid", "storage": "wifi.ssid", "access": "write"},
+					map[string]any{"request": "password", "storage": "wifi.psk", "access": "write"},
 				},
 			},
 		},
@@ -536,7 +537,7 @@ func (s *confdbSuite) SetUpTest(c *C) {
   }
 }`)
 
-	as, err := s.signingDB.Sign(asserts.ConfdbType, headers, body, "")
+	as, err := s.signingDB.Sign(asserts.ConfdbSchemaType, headers, body, "")
 	c.Assert(err, IsNil)
 	c.Assert(assertstate.Add(s.state, as), IsNil)
 
@@ -600,20 +601,23 @@ slots:
 
 func (s *confdbSuite) setConfdbFlag(val bool, c *C) {
 	tr := config.NewTransaction(s.state)
-	_, confOption := features.Confdbs.ConfigOption()
+	_, confOption := features.Confdb.ConfigOption()
 	err := tr.Set("core", confOption, val)
 	c.Assert(err, IsNil)
 	tr.Commit()
 }
 
 func (s *confdbSuite) TestConfdbGetSingleView(c *C) {
-	restore := ctlcmd.MockConfdbstateNewTransaction(func(st *state.State, account string, confdbName string) (*confdbstate.Transaction, error) {
-		c.Assert(account, Equals, s.devAccID)
-		c.Assert(confdbName, Equals, "network")
-
-		tx, _ := confdbstate.NewTransaction(st, account, confdbName)
-		c.Assert(tx.Set("wifi.ssid", "my-ssid"), IsNil)
+	s.state.Lock()
+	tx, err := confdbstate.NewTransaction(s.state, s.devAccID, "network")
+	c.Assert(err, IsNil)
+	c.Assert(tx.Set("wifi.ssid", "my-ssid"), IsNil)
+	s.state.Unlock()
 
+	restore := ctlcmd.MockConfdbstateTransactionForGet(func(ctx *hookstate.Context, view *confdb.View, requests []string) (*confdbstate.Transaction, error) {
+		c.Assert(requests, DeepEquals, []string{"ssid"})
+		c.Assert(view.Schema().Account, Equals, s.devAccID)
+		c.Assert(view.Schema().Name, Equals, "network")
 		return tx, nil
 	})
 	defer restore()
@@ -625,14 +629,17 @@ func (s *confdbSuite) TestConfdbGetSingleView(c *C) {
 }
 
 func (s *confdbSuite) TestConfdbGetManyViews(c *C) {
-	restore := ctlcmd.MockConfdbstateNewTransaction(func(st *state.State, account string, confdbName string) (*confdbstate.Transaction, error) {
-		c.Assert(account, Equals, s.devAccID)
-		c.Assert(confdbName, Equals, "network")
-
-		tx, _ := confdbstate.NewTransaction(st, account, confdbName)
-		c.Assert(tx.Set("wifi.ssid", "my-ssid"), IsNil)
-		c.Assert(tx.Set("wifi.psk", "secret"), IsNil)
+	s.state.Lock()
+	tx, err := confdbstate.NewTransaction(s.state, s.devAccID, "network")
+	c.Assert(err, IsNil)
+	c.Assert(tx.Set("wifi.ssid", "my-ssid"), IsNil)
+	c.Assert(tx.Set("wifi.psk", "secret"), IsNil)
+	s.state.Unlock()
 
+	restore := ctlcmd.MockConfdbstateTransactionForGet(func(ctx *hookstate.Context, view *confdb.View, requests []string) (*confdbstate.Transaction, error) {
+		c.Assert(requests, DeepEquals, []string{"ssid", "password"})
+		c.Assert(view.Schema().Account, Equals, s.devAccID)
+		c.Assert(view.Schema().Name, Equals, "network")
 		return tx, nil
 	})
 	defer restore()
@@ -648,14 +655,17 @@ func (s *confdbSuite) TestConfdbGetManyViews(c *C) {
 }
 
 func (s *confdbSuite) TestConfdbGetNoRequest(c *C) {
-	restore := ctlcmd.MockConfdbstateNewTransaction(func(st *state.State, account string, confdbName string) (*confdbstate.Transaction, error) {
-		c.Assert(account, Equals, s.devAccID)
-		c.Assert(confdbName, Equals, "network")
-
-		tx, _ := confdbstate.NewTransaction(st, account, confdbName)
-		c.Assert(tx.Set("wifi.ssid", "my-ssid"), IsNil)
-		c.Assert(tx.Set("wifi.psk", "secret"), IsNil)
+	s.state.Lock()
+	tx, err := confdbstate.NewTransaction(s.state, s.devAccID, "network")
+	c.Assert(err, IsNil)
+	c.Assert(tx.Set("wifi.ssid", "my-ssid"), IsNil)
+	c.Assert(tx.Set("wifi.psk", "secret"), IsNil)
+	s.state.Unlock()
 
+	restore := ctlcmd.MockConfdbstateTransactionForGet(func(ctx *hookstate.Context, view *confdb.View, requests []string) (*confdbstate.Transaction, error) {
+		c.Assert(requests, IsNil)
+		c.Assert(view.Schema().Account, Equals, s.devAccID)
+		c.Assert(view.Schema().Name, Equals, "network")
 		return tx, nil
 	})
 	defer restore()
@@ -760,40 +770,16 @@ slots:
 	c.Check(stderr, IsNil)
 }
 
-func (s *confdbSuite) TestConfdbGetAndSetAssertionNotFound(c *C) {
-	storeSigning := assertstest.NewStoreStack("can0nical", nil)
-	db, err := asserts.OpenDatabase(&asserts.DatabaseConfig{
-		Backstore: asserts.NewMemoryBackstore(),
-		Trusted:   storeSigning.Trusted,
-	})
-	c.Assert(err, IsNil)
-	c.Assert(db.Add(storeSigning.StoreAccountKey("")), IsNil)
-
-	s.state.Lock()
-	assertstate.ReplaceDB(s.state, db)
-	s.state.Unlock()
-
-	stdout, stderr, err := ctlcmd.Run(s.mockContext, []string{"get", "--view", ":read-wifi"}, 0)
-	c.Assert(err, ErrorMatches, fmt.Sprintf("cannot find confdb %s/network: assertion not found", s.devAccID))
-	c.Check(stdout, IsNil)
-	c.Check(stderr, IsNil)
-
-	stdout, stderr, err = ctlcmd.Run(s.mockContext, []string{"set", "--view", ":write-wifi", "ssid=my-ssid"}, 0)
-	c.Assert(err, ErrorMatches, fmt.Sprintf("cannot find confdb %s/network: assertion not found", s.devAccID))
-	c.Check(stdout, IsNil)
-	c.Check(stderr, IsNil)
-}
-
 func (s *confdbSuite) TestConfdbGetAndSetViewNotFound(c *C) {
-	headers := map[string]interface{}{
+	headers := map[string]any{
 		"authority-id": s.devAccID,
 		"account-id":   s.devAccID,
 		"revision":     "1",
 		"name":         "network",
-		"views": map[string]interface{}{
-			"other": map[string]interface{}{
-				"rules": []interface{}{
-					map[string]interface{}{"request": "a", "storage": "a"},
+		"views": map[string]any{
+			"other": map[string]any{
+				"rules": []any{
+					map[string]any{"request": "a", "storage": "a"},
 				},
 			},
 		},
@@ -808,53 +794,72 @@ func (s *confdbSuite) TestConfdbGetAndSetViewNotFound(c *C) {
   }
 }`)
 
-	as, err := s.signingDB.Sign(asserts.ConfdbType, headers, body, "")
+	as, err := s.signingDB.Sign(asserts.ConfdbSchemaType, headers, body, "")
 	c.Assert(err, IsNil)
 	s.state.Lock()
 	c.Assert(assertstate.Add(s.state, as), IsNil)
 	s.state.Unlock()
 
 	stdout, stderr, err := ctlcmd.Run(s.mockContext, []string{"get", "--view", ":read-wifi"}, 0)
-	c.Assert(err, ErrorMatches, fmt.Sprintf("cannot find view \"read-wifi\" in confdb %s/network", s.devAccID))
+	c.Assert(err, ErrorMatches, fmt.Sprintf("cannot find view \"read-wifi\" in confdb schema %s/network", s.devAccID))
 	c.Check(stdout, IsNil)
 	c.Check(stderr, IsNil)
 
 	stdout, stderr, err = ctlcmd.Run(s.mockContext, []string{"set", "--view", ":write-wifi", "ssid=my-ssid"}, 0)
-	c.Assert(err, ErrorMatches, fmt.Sprintf("cannot find view \"write-wifi\" in confdb %s/network", s.devAccID))
+	c.Assert(err, ErrorMatches, fmt.Sprintf("cannot find view \"write-wifi\" in confdb schema %s/network", s.devAccID))
 	c.Check(stdout, IsNil)
 	c.Check(stderr, IsNil)
 }
 
-func (s *confdbSuite) TestConfdbGetPristine(c *C) {
-	restore := ctlcmd.MockConfdbstateGetStoredTransaction(func(*state.Task) (*confdbstate.Transaction, func(), error) {
-		tx, _ := confdbstate.NewTransaction(s.state, s.devAccID, "network")
-		c.Assert(tx.Set("wifi.ssid", "foo"), IsNil)
-		c.Assert(tx.Commit(s.state, confdb.NewJSONSchema()), IsNil)
+func (s *confdbSuite) TestConfdbGetPrevious(c *C) {
+	s.state.Lock()
+	tx, err := confdbstate.NewTransaction(s.state, s.devAccID, "network")
+	c.Assert(err, IsNil)
 
-		c.Assert(tx.Set("wifi.ssid", "bar"), IsNil)
-		return tx, func() {}, nil
+	c.Assert(tx.Set("wifi.ssid", "foo"), IsNil)
+	c.Assert(tx.Commit(s.state, confdb.NewJSONSchema()), IsNil)
+
+	tx, err = confdbstate.NewTransaction(s.state, s.devAccID, "network")
+	c.Assert(err, IsNil)
+	c.Assert(tx.Set("wifi.ssid", "bar"), IsNil)
+
+	restore := ctlcmd.MockConfdbstateTransactionForGet(func(*hookstate.Context, *confdb.View, []string) (*confdbstate.Transaction, error) {
+		return tx, nil
 	})
 	defer restore()
 
-	s.state.Lock()
-	defer s.state.Unlock()
 	task := s.state.NewTask("run-hook", "")
 	setup := &hookstate.HookSetup{Snap: "test-snap", Hook: "save-view-plug"}
 	ctx, err := hookstate.NewContext(task, s.state, setup, s.mockHandler, "")
 	c.Assert(err, IsNil)
-
 	s.state.Unlock()
-	defer s.state.Lock()
 
-	stdout, stderr, err := ctlcmd.Run(ctx, []string{"get", "--view", "--pristine", ":read-wifi", "ssid"}, 0)
+	// current transaction has uncommitted write "bar"
+	stdout, stderr, err := ctlcmd.Run(ctx, []string{"get", "--view", ":read-wifi", "ssid"}, 0)
+	c.Assert(err, IsNil)
+	c.Check(string(stdout), Equals, "bar\n")
+	c.Check(stderr, IsNil)
+
+	// but --previous show "foo"
+	stdout, stderr, err = ctlcmd.Run(ctx, []string{"get", "--view", "--previous", ":read-wifi", "ssid"}, 0)
 	c.Assert(err, IsNil)
 	c.Check(string(stdout), Equals, "foo\n")
 	c.Check(stderr, IsNil)
 
-	stdout, stderr, err = ctlcmd.Run(ctx, []string{"get", "--view", ":read-wifi", "ssid"}, 0)
+	s.state.Lock()
+	// simulate a commit and then a observe-view- hook
+	c.Assert(tx.Commit(s.state, confdb.NewJSONSchema()), IsNil)
+	setup = &hookstate.HookSetup{Snap: "test-snap", Hook: "observe-view-plug"}
+	ctx, err = hookstate.NewContext(task, s.state, setup, s.mockHandler, "")
 	c.Assert(err, IsNil)
-	c.Check(string(stdout), Equals, "bar\n")
+	s.state.Unlock()
+
+	// --previous in observe-view hook refers to pre-commit databag
+	stdout, stderr, err = ctlcmd.Run(ctx, []string{"get", "--view", "--previous", ":read-wifi", "ssid"}, 0)
+	c.Assert(err, IsNil)
+	c.Check(string(stdout), Equals, "foo\n")
 	c.Check(stderr, IsNil)
+
 }
 
 func (s *confdbSuite) TestConfdbGetDifferentViewThanOngoingTx(c *C) {
@@ -864,9 +869,6 @@ func (s *confdbSuite) TestConfdbGetDifferentViewThanOngoingTx(c *C) {
 	tx, err := confdbstate.NewTransaction(s.state, s.devAccID, "network")
 	c.Assert(err, IsNil)
 
-	err = tx.Set("wifi.ssid", "foo")
-	c.Assert(err, IsNil)
-
 	task := s.state.NewTask("run-hook", "")
 	setup := &hookstate.HookSetup{Snap: "test-snap", Hook: "save-view-plug"}
 	ctx, err := hookstate.NewContext(task, s.state, setup, s.mockHandler, "")
@@ -879,10 +881,10 @@ func (s *confdbSuite) TestConfdbGetDifferentViewThanOngoingTx(c *C) {
 	defer s.state.Lock()
 
 	restore := ctlcmd.MockConfdbstateGetView(func(st *state.State, account, confdbName, viewName string) (*confdb.View, error) {
-		reg, err := confdb.New(s.devAccID, "other", map[string]interface{}{
-			"other": map[string]interface{}{
-				"rules": []interface{}{
-					map[string]interface{}{"request": "ssid", "storage": "ssid"},
+		reg, err := confdb.NewSchema(s.devAccID, "other", map[string]any{
+			"other": map[string]any{
+				"rules": []any{
+					map[string]any{"request": "ssid", "storage": "ssid"},
 				},
 			},
 		}, confdb.NewJSONSchema())
@@ -892,8 +894,7 @@ func (s *confdbSuite) TestConfdbGetDifferentViewThanOngoingTx(c *C) {
 	defer restore()
 
 	stdout, stderr, err := ctlcmd.Run(ctx, []string{"get", "--view", ":other", "ssid"}, 0)
-	// error is for no stored value, meaning we read the right confdb
-	c.Assert(err, ErrorMatches, `.*: no view data`)
+	c.Assert(err, ErrorMatches, fmt.Sprintf(`cannot load confdb %[1]s/other: ongoing transaction for %[1]s/network`, s.devAccID))
 	c.Check(stdout, IsNil)
 	c.Check(stderr, IsNil)
 }
@@ -905,8 +906,78 @@ func (s *confdbSuite) TestConfdbExperimentalFlag(c *C) {
 
 	for _, cmd := range []string{"get", "set", "unset"} {
 		stdout, stderr, err := ctlcmd.Run(s.mockContext, []string{cmd, "--view", ":read-wifi"}, 0)
-		c.Assert(err, ErrorMatches, i18n.G(`"confdbs" feature flag is disabled: set 'experimental.confdbs' to true`))
+		c.Assert(err, ErrorMatches, i18n.G(`"confdb" feature flag is disabled: set 'experimental.confdb' to true`))
 		c.Check(stdout, IsNil)
 		c.Check(stderr, IsNil)
 	}
 }
+
+func (s *confdbSuite) TestConfdbGetPreviousInvalid(c *C) {
+	restore := confdbstate.MockFetchConfdbSchemaAssertion(func(*state.State, int, string, string) error {
+		return store.ErrStoreOffline
+	})
+	defer restore()
+
+	// the parsing succeeded
+	success := fmt.Sprintf(`confdb-schema (other; account-id:%s) not found`, s.devAccID)
+	forbidMsg := `cannot use --previous outside of save-view, change-view or observe-view hooks`
+
+	type testcase struct {
+		hook string
+		err  string
+	}
+
+	tcs := []testcase{
+		{
+			hook: "save-view-plug",
+			err:  success,
+		},
+		{
+			hook: "load-view-plug",
+			err:  forbidMsg,
+		},
+		{
+			hook: "change-view-plug",
+			err:  success,
+		},
+		{
+			hook: "query-view-pug",
+			err:  forbidMsg,
+		},
+		{
+			hook: "observe-view-plug",
+			err:  success,
+		},
+		{
+			hook: "",
+			err:  forbidMsg,
+		},
+	}
+
+	for _, tc := range tcs {
+		var ctx *hookstate.Context
+		var err error
+		if tc.hook != "" {
+			s.state.Lock()
+			task := s.state.NewTask("run-hook", "hook-task")
+			setup := &hookstate.HookSetup{Snap: "test-snap", Revision: snap.R(1), Hook: tc.hook}
+			s.state.Unlock()
+
+			ctx, err = hookstate.NewContext(task, s.state, setup, s.mockHandler, "")
+			c.Assert(err, IsNil)
+		} else {
+			ctx, err = hookstate.NewContext(nil, s.state, nil, s.mockHandler, "")
+			c.Assert(err, IsNil)
+		}
+
+		stdout, stderr, err := ctlcmd.Run(ctx, []string{"get", "--view", "--previous", ":other", "foo"}, 0)
+		c.Assert(err.Error(), Equals, tc.err)
+		c.Check(stdout, IsNil)
+		c.Check(stderr, IsNil)
+	}
+
+	stdout, stderr, err := ctlcmd.Run(s.mockContext, []string{"get", "--previous", ":other", "foo"}, 0)
+	c.Assert(err.Error(), Equals, "cannot use --previous without --view")
+	c.Check(stdout, IsNil)
+	c.Check(stderr, IsNil)
+}
