@@ -66,6 +66,7 @@ type fakeOp struct {
 	componentRev                    snap.Revision
 	componentSideInfo               snap.ComponentSideInfo
 	componentSkipAssertionsDownload bool
+	componentRemoveOpts             backend.RemoveComponentOpts
 
 	curSnaps []store.CurrentSnap
 	action   store.SnapAction
@@ -154,6 +155,12 @@ type fakeDownload struct {
 	opts     *store.DownloadOptions
 }
 
+type fakeIconDownload struct {
+	name   string
+	target string
+	url    string
+}
+
 type byName []store.CurrentSnap
 
 func (bna byName) Len() int      { return len(bna) }
@@ -183,6 +190,7 @@ type fakeStore struct {
 	mu sync.Mutex
 
 	downloads           []fakeDownload
+	iconDownloads       []fakeIconDownload
 	refreshRevnos       map[string]snap.Revision
 	fakeBackend         *fakeSnappyBackend
 	fakeCurrentProgress int
@@ -196,7 +204,8 @@ type fakeStore struct {
 	// it should return the resources that the snap should have.
 	snapResourcesFn func(*snap.Info) []store.SnapResourceResult
 
-	downloadCallback func()
+	downloadCallback     func()
+	downloadIconCallback func(targetPath string)
 
 	namesToAssertedIDs map[string]string
 	idsToNames         map[string]string
@@ -250,6 +259,12 @@ func (f *fakeStore) appendDownload(dl *fakeDownload) {
 	f.downloads = append(f.downloads, *dl)
 }
 
+func (f *fakeStore) appendIconDownload(dl *fakeIconDownload) {
+	f.mu.Lock()
+	defer f.mu.Unlock()
+	f.iconDownloads = append(f.iconDownloads, *dl)
+}
+
 type snapSpec struct {
 	Name     string
 	Channel  string
@@ -376,7 +391,7 @@ func (f *fakeStore) snap(spec snapSpec) (*snap.Info, error) {
 			Snap:      info,
 			Name:      "dbus-slot",
 			Interface: "dbus",
-			Attrs: map[string]interface{}{
+			Attrs: map[string]any{
 				"bus":  "system",
 				"name": "org.example.Foo",
 			},
@@ -401,7 +416,7 @@ func (f *fakeStore) snap(spec snapSpec) (*snap.Info, error) {
 				Snap:      info,
 				Interface: "confdb",
 				Name:      "my-plug",
-				Attrs: map[string]interface{}{
+				Attrs: map[string]any{
 					"account": "my-publisher",
 					"view":    "my-reg/my-view",
 				},
@@ -427,8 +442,8 @@ func (f *fakeStore) snap(spec snapSpec) (*snap.Info, error) {
 				Snap:      info,
 				Interface: "desktop",
 				Name:      "desktop",
-				Attrs: map[string]interface{}{
-					"desktop-file-ids": []interface{}{"org.example.Foo"},
+				Attrs: map[string]any{
+					"desktop-file-ids": []any{"org.example.Foo"},
 				},
 			},
 		}
@@ -628,7 +643,7 @@ func (f *fakeStore) lookupRefresh(cand refreshCand) (*snap.Info, error) {
 			"content-plug": {
 				Snap:      info,
 				Interface: "content",
-				Attrs: map[string]interface{}{
+				Attrs: map[string]any{
 					"content":          "some-content",
 					"default-provider": "outdated-producer",
 				},
@@ -639,7 +654,7 @@ func (f *fakeStore) lookupRefresh(cand refreshCand) (*snap.Info, error) {
 			"content-slot": {
 				Snap:      info,
 				Interface: "content",
-				Attrs:     map[string]interface{}{"content": "some-content"},
+				Attrs:     map[string]any{"content": "some-content"},
 			},
 		}
 	} else if name == "provenance-snap" {
@@ -666,7 +681,7 @@ func (f *fakeStore) lookupRefresh(cand refreshCand) (*snap.Info, error) {
 				Snap:      info,
 				Interface: "confdb",
 				Name:      "my-plug",
-				Attrs: map[string]interface{}{
+				Attrs: map[string]any{
 					"account": "my-publisher",
 					"view":    "my-reg/my-view",
 				},
@@ -931,6 +946,23 @@ func (f *fakeStore) Download(ctx context.Context, name, targetFn string, snapInf
 	return nil
 }
 
+func (f *fakeStore) DownloadIcon(ctx context.Context, name string, targetPath string, downloadURL string) error {
+	// we don't want to hold state lock while waiting on the icon download
+	f.pokeStateLock()
+
+	if f.downloadIconCallback != nil {
+		f.downloadIconCallback(targetPath)
+	}
+	f.appendIconDownload(&fakeIconDownload{
+		name:   name,
+		target: targetPath,
+		url:    downloadURL,
+	})
+	f.fakeBackend.appendOp(&fakeOp{op: "storesvc-download-icon", name: name})
+
+	return nil
+}
+
 func (f *fakeStore) WriteCatalogs(ctx context.Context, _ io.Writer, _ store.SnapAdder) error {
 	if ctx == nil {
 		panic("context required")
@@ -1129,12 +1161,13 @@ func (f *fakeSnappyBackend) SetupKernelModulesComponents(currentComps, finalComp
 	return f.maybeErrForLastOp()
 }
 
-func (f *fakeSnappyBackend) UndoSetupComponent(cpi snap.ContainerPlaceInfo, installRecord *backend.InstallRecord, dev snap.Device, meter progress.Meter) error {
+func (f *fakeSnappyBackend) UndoSetupComponent(cpi snap.ContainerPlaceInfo, installRecord *backend.InstallRecord, dev snap.Device, removeOpts backend.RemoveComponentOpts, meter progress.Meter) error {
 	meter.Notify("undo-setup-component")
 	f.appendOp(&fakeOp{
-		op:                "undo-setup-component",
-		containerName:     cpi.ContainerName(),
-		containerFileName: cpi.Filename(),
+		op:                  "undo-setup-component",
+		containerName:       cpi.ContainerName(),
+		containerFileName:   cpi.Filename(),
+		componentRemoveOpts: removeOpts,
 	})
 	if strings.HasSuffix(cpi.ContainerName(), "+brokenundo") {
 		return fmt.Errorf("cannot undo set-up of component %q", cpi.ContainerName())
