@@ -25,10 +25,12 @@ import (
 	"github.com/snapcore/snapd/boot"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/overlord/state"
-	"github.com/snapcore/snapd/release"
+	"github.com/snapcore/snapd/overlord/swfeats"
 	"github.com/snapcore/snapd/snap"
 )
 
+var updateRevisionsChangeKind = swfeats.RegisterChangeKind("update-revisions")
+
 // UpdateBootRevisions synchronizes the active kernel and OS snap versions
 // with the versions that actually booted. This is needed because a
 // system may install "os=v2" but that fails to boot. The bootloader
@@ -40,10 +42,6 @@ import (
 func UpdateBootRevisions(st *state.State) error {
 	const errorPrefix = "cannot update revisions after boot changes: "
 
-	if release.OnClassic {
-		return nil
-	}
-
 	// nothing to check if there's no kernel
 	ok, err := HasSnapOfType(st, snap.TypeKernel)
 	if err != nil {
@@ -59,17 +57,16 @@ func UpdateBootRevisions(st *state.State) error {
 		return err
 	}
 
-	kernel, err := boot.GetCurrentBoot(snap.TypeKernel, deviceCtx)
-	if err != nil {
-		return fmt.Errorf(errorPrefix+"%s", err)
-	}
-	base, err := boot.GetCurrentBoot(snap.TypeBase, deviceCtx)
-	if err != nil {
-		return fmt.Errorf(errorPrefix+"%s", err)
-	}
-
 	var tsAll []*state.TaskSet
-	for _, actual := range []snap.PlaceInfo{kernel, base} {
+	for _, typ := range []snap.Type{snap.TypeKernel, snap.TypeBase} {
+		if !boot.SnapTypeParticipatesInBoot(typ, deviceCtx) {
+			continue
+		}
+
+		actual, err := boot.GetCurrentBoot(typ, deviceCtx)
+		if err != nil {
+			return fmt.Errorf(errorPrefix+"%s", err)
+		}
 		info, err := CurrentInfo(st, actual.SnapName())
 		if err != nil {
 			logger.Noticef("cannot get info for %q: %s", actual.SnapName(), err)
@@ -91,7 +88,7 @@ func UpdateBootRevisions(st *state.State) error {
 	}
 
 	msg := "Update kernel and core snap revisions"
-	chg := st.NewChange("update-revisions", msg)
+	chg := st.NewChange(updateRevisionsChangeKind, msg)
 	for _, ts := range tsAll {
 		chg.AddAll(ts)
 	}
