@@ -73,9 +73,9 @@ func MockSnapReadInfo(mock func(name string, si *snap.SideInfo) (*snap.Info, err
 }
 
 func MockReadComponentInfo(mock func(compMntDir string, snapInfo *snap.Info, csi *snap.ComponentSideInfo) (*snap.ComponentInfo, error)) (restore func()) {
-	old := readComponentInfo
-	readComponentInfo = mock
-	return func() { readComponentInfo = old }
+	old := readComponentInfoAt
+	readComponentInfoAt = mock
+	return func() { readComponentInfoAt = old }
 }
 
 func MockMountPollInterval(intv time.Duration) (restore func()) {
@@ -171,6 +171,11 @@ var (
 	CheckDBusServiceConflicts = checkDBusServiceConflicts
 )
 
+// desktop-file-ids
+var (
+	CheckDesktopFileIDsConflicts = checkDesktopFileIDsConflicts
+)
+
 // readme files
 var (
 	WriteSnapReadme = writeSnapReadme
@@ -327,15 +332,7 @@ func MockReRefreshRetryTimeout(d time.Duration) (restore func()) {
 	}
 }
 
-// aux store info
-var (
-	AuxStoreInfoFilename = auxStoreInfoFilename
-	RetrieveAuxStoreInfo = retrieveAuxStoreInfo
-	KeepAuxStoreInfo     = keepAuxStoreInfo
-	DiscardAuxStoreInfo  = discardAuxStoreInfo
-)
-
-type AuxStoreInfo = auxStoreInfo
+type DisabledServices = disabledServices
 
 // link, misc handlers
 var (
