@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2016 Canonical Ltd
+ * Copyright (C) 2014-2021 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -20,17 +20,19 @@
 package snap_test
 
 import (
+	"fmt"
 	"regexp"
+	"sort"
 	"testing"
 	"time"
 
 	. "gopkg.in/check.v1"
 
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snap/naming"
 	"github.com/snapcore/snapd/strutil"
-	"github.com/snapcore/snapd/timeout"
-
 	"github.com/snapcore/snapd/testutil"
+	"github.com/snapcore/snapd/timeout"
 )
 
 // Hook up check.v1 into the "go test" runner
@@ -61,8 +63,10 @@ func (s *InfoSnapYamlTestSuite) TestSimple(c *C) {
 	c.Assert(err, IsNil)
 	c.Assert(info.InstanceName(), Equals, "foo")
 	c.Assert(info.Version, Equals, "1.0")
-	c.Assert(info.Type, Equals, snap.TypeApp)
+	c.Assert(info.Type(), Equals, snap.TypeApp)
 	c.Assert(info.Epoch, DeepEquals, snap.E("0"))
+	c.Assert(info.SnapProvenance, Equals, "")
+	c.Check(info.Provenance(), Equals, naming.DefaultProvenance)
 }
 
 func (s *InfoSnapYamlTestSuite) TestSnapdTypeAddedByMagic(c *C) {
@@ -71,7 +75,16 @@ version: 1.0`))
 	c.Assert(err, IsNil)
 	c.Assert(info.InstanceName(), Equals, "snapd")
 	c.Assert(info.Version, Equals, "1.0")
-	c.Assert(info.Type, Equals, snap.TypeSnapd)
+	c.Assert(info.Type(), Equals, snap.TypeSnapd)
+}
+
+func (s *InfoSnapYamlTestSuite) TestNonDefaultProvenance(c *C) {
+	info, err := snap.InfoFromSnapYaml([]byte(`name: foo
+provenance: delegated-prov
+version: 1.0`))
+	c.Assert(err, IsNil)
+	c.Check(info.Provenance(), Equals, "delegated-prov")
+	c.Check(info.SnapProvenance, Equals, "delegated-prov")
 }
 
 func (s *InfoSnapYamlTestSuite) TestFail(c *C) {
@@ -128,6 +141,7 @@ plugs:
 		Snap:      info,
 		Name:      "network-client",
 		Interface: "network-client",
+		Unscoped:  true,
 	})
 }
 
@@ -146,6 +160,7 @@ plugs:
 		Snap:      info,
 		Name:      "net",
 		Interface: "network-client",
+		Unscoped:  true,
 	})
 }
 
@@ -165,6 +180,7 @@ plugs:
 		Snap:      info,
 		Name:      "net",
 		Interface: "network-client",
+		Unscoped:  true,
 	})
 }
 
@@ -185,7 +201,8 @@ plugs:
 		Snap:      info,
 		Name:      "net",
 		Interface: "network-client",
-		Attrs:     map[string]interface{}{"ipv6-aware": true},
+		Attrs:     map[string]any{"ipv6-aware": true},
+		Unscoped:  true,
 	})
 }
 
@@ -210,11 +227,12 @@ plugs:
 		Snap:      info,
 		Name:      "iface",
 		Interface: "complex",
-		Attrs: map[string]interface{}{
+		Attrs: map[string]any{
 			"i": int64(3),
-			"l": []interface{}{int64(1), int64(2), int64(3)},
-			"m": map[string]interface{}{"a": "A", "b": "B"},
+			"l": []any{int64(1), int64(2), int64(3)},
+			"m": map[string]any{"a": "A", "b": "B"},
 		},
+		Unscoped: true,
 	})
 }
 
@@ -238,7 +256,8 @@ plugs:
 		Snap:      info,
 		Name:      "net",
 		Interface: "network-client",
-		Attrs:     map[string]interface{}{"attr": int64(2)},
+		Attrs:     map[string]any{"attr": int64(2)},
+		Unscoped:  true,
 	})
 }
 
@@ -264,6 +283,7 @@ apps:
 		Name:      "network-client",
 		Interface: "network-client",
 		Apps:      map[string]*snap.AppInfo{app.Name: app},
+		Unscoped:  true,
 	})
 	c.Assert(app, DeepEquals, &snap.AppInfo{
 		Snap:  info,
@@ -385,7 +405,8 @@ plugs:
 		Snap:      info,
 		Name:      "network-client",
 		Interface: "network-client",
-		Attrs:     map[string]interface{}{"ipv6-aware": true},
+		Attrs:     map[string]any{"ipv6-aware": true},
+		Unscoped:  true,
 	})
 }
 
@@ -407,6 +428,7 @@ plugs:
 		Name:      "bool-file",
 		Interface: "bool-file",
 		Label:     "Disk I/O indicator",
+		Unscoped:  true,
 	})
 }
 
@@ -441,7 +463,18 @@ plugs:
     net:
         1: ok
 `))
-	c.Assert(err, ErrorMatches, `plug "net" has attribute that is not a string \(found int\)`)
+	c.Assert(err, ErrorMatches, `plug "net" has attribute key that is not a string \(found int\)`)
+}
+
+func (s *YamlSuite) TestUnmarshalCorruptedPlugWithEmptyAttributeKey(c *C) {
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	_, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+plugs:
+    net:
+        "": ok
+`))
+	c.Assert(err, ErrorMatches, `plug "net" has an empty attribute key`)
 }
 
 func (s *YamlSuite) TestUnmarshalCorruptedPlugWithUnexpectedType(c *C) {
@@ -509,6 +542,7 @@ slots:
 		Snap:      info,
 		Name:      "network-client",
 		Interface: "network-client",
+		Unscoped:  true,
 	})
 }
 
@@ -527,6 +561,7 @@ slots:
 		Snap:      info,
 		Name:      "net",
 		Interface: "network-client",
+		Unscoped:  true,
 	})
 }
 
@@ -546,6 +581,7 @@ slots:
 		Snap:      info,
 		Name:      "net",
 		Interface: "network-client",
+		Unscoped:  true,
 	})
 }
 
@@ -566,7 +602,8 @@ slots:
 		Snap:      info,
 		Name:      "net",
 		Interface: "network-client",
-		Attrs:     map[string]interface{}{"ipv6-aware": true},
+		Attrs:     map[string]any{"ipv6-aware": true},
+		Unscoped:  true,
 	})
 }
 
@@ -590,11 +627,12 @@ slots:
 		Snap:      info,
 		Name:      "iface",
 		Interface: "complex",
-		Attrs: map[string]interface{}{
+		Attrs: map[string]any{
 			"i": int64(3),
-			"l": []interface{}{int64(1), int64(2)},
-			"m": map[string]interface{}{"a": "A"},
+			"l": []any{int64(1), int64(2)},
+			"m": map[string]any{"a": "A"},
 		},
+		Unscoped: true,
 	})
 }
 
@@ -618,7 +656,8 @@ slots:
 		Snap:      info,
 		Name:      "net",
 		Interface: "network-client",
-		Attrs:     map[string]interface{}{"attr": int64(2)},
+		Attrs:     map[string]any{"attr": int64(2)},
+		Unscoped:  true,
 	})
 }
 
@@ -643,6 +682,7 @@ apps:
 		Name:      "network-client",
 		Interface: "network-client",
 		Apps:      map[string]*snap.AppInfo{app.Name: app},
+		Unscoped:  true,
 	})
 	c.Assert(app, DeepEquals, &snap.AppInfo{
 		Snap:  info,
@@ -726,7 +766,8 @@ slots:
 		Snap:      info,
 		Name:      "network-client",
 		Interface: "network-client",
-		Attrs:     map[string]interface{}{"ipv6-aware": true},
+		Attrs:     map[string]any{"ipv6-aware": true},
+		Unscoped:  true,
 	})
 }
 
@@ -749,6 +790,7 @@ slots:
 		Name:      "led0",
 		Interface: "bool-file",
 		Label:     "Front panel LED (red)",
+		Unscoped:  true,
 	})
 }
 
@@ -777,7 +819,7 @@ hooks:
 		Snap:      info,
 		Name:      "test-slot",
 		Interface: "test-slot",
-		Hooks:     map[string]*snap.HookInfo{hook.Name: hook},
+		Unscoped:  true,
 	})
 	c.Check(hook, DeepEquals, &snap.HookInfo{
 		Snap:  info,
@@ -812,7 +854,6 @@ hooks:
 		Snap:      info,
 		Name:      "test-slot",
 		Interface: "test-slot",
-		Hooks:     map[string]*snap.HookInfo{hook.Name: hook},
 	})
 	c.Check(hook, DeepEquals, &snap.HookInfo{
 		Snap:  info,
@@ -854,7 +895,18 @@ slots:
     net:
         1: ok
 `))
-	c.Assert(err, ErrorMatches, `slot "net" has attribute that is not a string \(found int\)`)
+	c.Assert(err, ErrorMatches, `slot "net" has attribute key that is not a string \(found int\)`)
+}
+
+func (s *YamlSuite) TestUnmarshalCorruptedSlotWithEmptyAttributeKey(c *C) {
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	_, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+slots:
+    net:
+        "": ok
+`))
+	c.Assert(err, ErrorMatches, `slot "net" has an empty attribute key`)
 }
 
 func (s *YamlSuite) TestUnmarshalCorruptedSlotWithUnexpectedType(c *C) {
@@ -991,7 +1043,6 @@ hooks:
 		Snap:      info,
 		Name:      "test-plug",
 		Interface: "test-plug",
-		Hooks:     map[string]*snap.HookInfo{hook.Name: hook},
 	})
 	c.Check(hook, DeepEquals, &snap.HookInfo{
 		Snap:  info,
@@ -1027,7 +1078,7 @@ hooks:
 		Snap:      info,
 		Name:      "test-plug",
 		Interface: "test-plug",
-		Hooks:     map[string]*snap.HookInfo{hook.Name: hook},
+		Unscoped:  true,
 	})
 	c.Check(hook, DeepEquals, &snap.HookInfo{
 		Snap:  info,
@@ -1063,7 +1114,6 @@ hooks:
 		Snap:      info,
 		Name:      "test-plug",
 		Interface: "test-plug",
-		Hooks:     map[string]*snap.HookInfo{withPlugHook.Name: withPlugHook},
 	})
 	c.Assert(withPlugHook, DeepEquals, &snap.HookInfo{
 		Snap:  info,
@@ -1107,7 +1157,6 @@ hooks:
 		Snap:      info,
 		Name:      "test-plug",
 		Interface: "test-interface",
-		Hooks:     map[string]*snap.HookInfo{hook.Name: hook},
 	})
 	c.Check(hook, DeepEquals, &snap.HookInfo{
 		Snap:  info,
@@ -1145,7 +1194,6 @@ apps:
 		Name:      "test-plug",
 		Interface: "test-plug",
 		Apps:      map[string]*snap.AppInfo{},
-		Hooks:     map[string]*snap.HookInfo{hook.Name: hook},
 	})
 	c.Assert(hook, DeepEquals, &snap.HookInfo{
 		Snap:  info,
@@ -1203,7 +1251,7 @@ slots:
 	c.Assert(err, IsNil)
 	c.Check(info.InstanceName(), Equals, "foo")
 	c.Check(info.Version, Equals, "1.2")
-	c.Check(info.Type, Equals, snap.TypeApp)
+	c.Check(info.Type(), Equals, snap.TypeApp)
 	c.Check(info.Epoch, DeepEquals, snap.E("1*"))
 	c.Check(info.Confinement, Equals, snap.DevModeConfinement)
 	c.Check(info.Title(), Equals, "Foo")
@@ -1311,7 +1359,7 @@ slots:
 	c.Check(slot1.Snap, Equals, info)
 	c.Check(slot1.Name, Equals, "foo-socket-slot")
 	c.Check(slot1.Interface, Equals, "socket")
-	c.Check(slot1.Attrs, DeepEquals, map[string]interface{}{
+	c.Check(slot1.Attrs, DeepEquals, map[string]any{
 		"protocol": "foo", "path": "$SNAP_DATA/socket"})
 	c.Check(slot1.Label, Equals, "")
 	c.Check(slot1.Apps, DeepEquals, map[string]*snap.AppInfo{app1.Name: app1})
@@ -1328,6 +1376,64 @@ slots:
 		app1.Name: app1, app2.Name: app2})
 }
 
+func (s *YamlSuite) TestUnmarshalActivatesOn(c *C) {
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+slots:
+    test-slot1:
+    test-slot2:
+apps:
+    daemon:
+        activates-on: ["test-slot1", "test-slot2"]
+    foo:
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.InstanceName(), Equals, "snap")
+	c.Check(info.Plugs, HasLen, 0)
+	c.Check(info.Slots, HasLen, 2)
+	c.Check(info.Apps, HasLen, 2)
+	c.Check(info.Hooks, HasLen, 0)
+
+	app1 := info.Apps["daemon"]
+	app2 := info.Apps["foo"]
+	slot1 := info.Slots["test-slot1"]
+	slot2 := info.Slots["test-slot2"]
+
+	c.Assert(app1, Not(IsNil))
+	c.Check(app1.Name, Equals, "daemon")
+	c.Check(app1.ActivatesOn, DeepEquals, []*snap.SlotInfo{slot1, slot2})
+	// activates-on slots are implicitly added to the app
+	c.Check(app1.Slots, DeepEquals, map[string]*snap.SlotInfo{
+		slot1.Name: slot1, slot2.Name: slot2})
+
+	c.Assert(app2, Not(IsNil))
+	c.Check(app2.Name, Equals, "foo")
+	c.Check(app2.ActivatesOn, HasLen, 0)
+	// As slot has been bound to app1, it isn't implicitly applied here
+	c.Check(app2.Slots, HasLen, 0)
+
+	c.Assert(slot1, Not(IsNil))
+	c.Check(slot1.Name, Equals, "test-slot1")
+	c.Check(slot1.Apps, DeepEquals, map[string]*snap.AppInfo{
+		app1.Name: app1})
+
+	c.Assert(slot2, Not(IsNil))
+	c.Check(slot2.Name, Equals, "test-slot2")
+	c.Check(slot2.Apps, DeepEquals, map[string]*snap.AppInfo{
+		app1.Name: app1})
+}
+
+func (s *YamlSuite) TestUnmarshalActivatesOnUnknownSlot(c *C) {
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+apps:
+    daemon:
+        activates-on: ["test-slot"]
+`))
+	c.Check(info, IsNil)
+	c.Check(err, ErrorMatches, `invalid activates-on value "test-slot" on app "daemon": slot not found`)
+}
+
 // type and architectures
 
 func (s *YamlSuite) TestSnapYamlTypeDefault(c *C) {
@@ -1336,7 +1442,7 @@ version: 1.0
 `)
 	info, err := snap.InfoFromSnapYaml(y)
 	c.Assert(err, IsNil)
-	c.Assert(info.Type, Equals, snap.TypeApp)
+	c.Assert(info.Type(), Equals, snap.TypeApp)
 }
 
 func (s *YamlSuite) TestSnapYamlEpochDefault(c *C) {
@@ -1407,18 +1513,6 @@ architectures:
 	c.Assert(err, NotNil)
 }
 
-func (s *YamlSuite) TestSnapYamlLicenseParsing(c *C) {
-	y := []byte(`
-name: foo
-version: 1.0
-license-agreement: explicit
-license-version: 12`)
-	info, err := snap.InfoFromSnapYaml(y)
-	c.Assert(err, IsNil)
-	c.Assert(info.LicenseAgreement, Equals, "explicit")
-	c.Assert(info.LicenseVersion, Equals, "12")
-}
-
 // apps
 
 func (s *YamlSuite) TestSimpleAppExample(c *C) {
@@ -1447,7 +1541,9 @@ apps:
    command: svc1
    description: svc one
    stop-timeout: 25s
+   start-timeout: 42m
    daemon: forking
+   daemon-scope: system
    stop-command: stop-cmd
    post-stop-command: post-stop-cmd
    restart-condition: on-abnormal
@@ -1465,8 +1561,10 @@ apps:
 		Name:            "svc",
 		Command:         "svc1",
 		Daemon:          "forking",
+		DaemonScope:     snap.SystemDaemon,
 		RestartCond:     snap.RestartOnAbnormal,
 		StopTimeout:     timeout.Timeout(25 * time.Second),
+		StartTimeout:    timeout.Timeout(42 * time.Minute),
 		StopCommand:     "stop-cmd",
 		PostStopCommand: "post-stop-cmd",
 		BusName:         "busName",
@@ -1483,6 +1581,35 @@ apps:
 	c.Check(info.Apps, DeepEquals, map[string]*snap.AppInfo{"svc": &app})
 }
 
+func (s *YamlSuite) TestDaemonUserDaemon(c *C) {
+	y := []byte(`name: wat
+version: 42
+apps:
+ svc:
+   command: svc1
+   daemon: simple
+   daemon-scope: user
+`)
+	info, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, IsNil)
+	c.Check(info.Apps["svc"].DaemonScope, Equals, snap.UserDaemon)
+}
+
+func (s *YamlSuite) TestDaemonNoDaemonScope(c *C) {
+	y := []byte(`name: wat
+version: 42
+apps:
+ svc:
+   command: svc1
+   daemon: simple
+`)
+	info, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, IsNil)
+
+	// If daemon-scope is unset, default to system scope
+	c.Check(info.Apps["svc"].DaemonScope, Equals, snap.SystemDaemon)
+}
+
 func (s *YamlSuite) TestDaemonListenStreamAsInteger(c *C) {
 	y := []byte(`name: wat
 version: 42
@@ -1526,8 +1653,20 @@ apps:
        socket-mode: asdfasdf
 `)
 	_, err := snap.InfoFromSnapYaml(y)
-	c.Check(err.Error(), Equals, "cannot parse snap.yaml: yaml: unmarshal errors:\n"+
-		"  line 9: cannot unmarshal !!str `asdfasdf` into os.FileMode")
+	c.Check(err.Error(), Matches, "cannot parse snap.yaml: yaml: unmarshal errors:\n"+
+		"  line 9: cannot unmarshal !!str `asdfasdf` into (os|fs).FileMode")
+}
+
+func (s *YamlSuite) TestDaemonInvalidDaemonScope(c *C) {
+	y := []byte(`name: wat
+version: 42
+apps:
+ svc:
+   command: svc
+   daemon-scope: invalid
+`)
+	_, err := snap.InfoFromSnapYaml(y)
+	c.Check(err.Error(), Equals, "cannot parse snap.yaml: invalid daemon scope: \"invalid\"")
 }
 
 func (s *YamlSuite) TestSnapYamlGlobalEnvironment(c *C) {
@@ -1829,3 +1968,395 @@ apps:
 	c.Assert(app, NotNil)
 	c.Check(app.RestartDelay, Equals, timeout.Timeout(12*time.Second))
 }
+
+func (s *YamlSuite) TestSnapYamlSystemUsernamesParsing(c *C) {
+	y := []byte(`name: binary
+version: 1.0
+system-usernames:
+  foo: shared
+  bar:
+    scope: external
+  baz:
+    scope: private
+    attr1: norf
+    attr2: corge
+    attr3: ""
+`)
+	info, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, IsNil)
+	c.Check(info.SystemUsernames, HasLen, 3)
+	c.Assert(info.SystemUsernames["foo"], DeepEquals, &snap.SystemUsernameInfo{
+		Name:  "foo",
+		Scope: "shared",
+	})
+	c.Assert(info.SystemUsernames["bar"], DeepEquals, &snap.SystemUsernameInfo{
+		Name:  "bar",
+		Scope: "external",
+	})
+	c.Assert(info.SystemUsernames["baz"], DeepEquals, &snap.SystemUsernameInfo{
+		Name:  "baz",
+		Scope: "private",
+		Attrs: map[string]any{
+			"attr1": "norf",
+			"attr2": "corge",
+			"attr3": "",
+		},
+	})
+}
+
+func (s *YamlSuite) TestSnapYamlSystemUsernamesParsingBadType(c *C) {
+	y := []byte(`name: binary
+version: 1.0
+system-usernames:
+  a: true
+`)
+	info, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, ErrorMatches, `system username "a" has malformed definition \(found bool\)`)
+	c.Assert(info, IsNil)
+}
+
+func (s *YamlSuite) TestSnapYamlSystemUsernamesParsingBadValue(c *C) {
+	y := []byte(`name: binary
+version: 1.0
+system-usernames:
+  a: [b, c]
+`)
+	info, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, ErrorMatches, `system username "a" has malformed definition \(found \[\]interface {}\)`)
+	c.Assert(info, IsNil)
+}
+
+func (s *YamlSuite) TestSnapYamlSystemUsernamesParsingBadKeyEmpty(c *C) {
+	y := []byte(`name: binary
+version: 1.0
+system-usernames:
+  "": shared
+`)
+	_, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, ErrorMatches, `system username cannot be empty`)
+}
+
+func (s *YamlSuite) TestSnapYamlSystemUsernamesParsingBadKeyList(c *C) {
+	y := []byte(`name: binary
+version: 1.0
+system-usernames:
+- foo: shared
+`)
+	_, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, ErrorMatches, `(?m)cannot parse snap.yaml:.*`)
+}
+
+func (s *YamlSuite) TestSnapYamlSystemUsernamesParsingBadValueEmpty(c *C) {
+	y := []byte(`name: binary
+version: 1.0
+system-usernames:
+  a: ""
+`)
+	_, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, ErrorMatches, `system username "a" does not specify a scope`)
+}
+
+func (s *YamlSuite) TestSnapYamlSystemUsernamesParsingBadValueNull(c *C) {
+	y := []byte(`name: binary
+version: 1.0
+system-usernames:
+  a: null
+`)
+	_, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, ErrorMatches, `system username "a" does not specify a scope`)
+}
+
+func (s *YamlSuite) TestSnapYamlSystemUsernamesParsingBadAttrKeyEmpty(c *C) {
+	y := []byte(`name: binary
+version: 1.0
+system-usernames:
+  foo:
+    scope: shared
+    "": bar
+`)
+	_, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, ErrorMatches, `system username "foo" has an empty attribute key`)
+}
+
+func (s *YamlSuite) TestSnapYamlSystemUsernamesParsingBadAttrKeyNonString(c *C) {
+	y := []byte(`name: binary
+version: 1.0
+system-usernames:
+  foo:
+    scope: shared
+    1: bar
+`)
+	_, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, ErrorMatches, `system username "foo" has attribute key that is not a string \(found int\)`)
+}
+
+func (s *YamlSuite) TestSnapYamlSystemUsernamesParsingBadAttrValue(c *C) {
+	y := []byte(`name: binary
+version: 1.0
+system-usernames:
+  foo:
+    scope: shared
+    bar: null
+`)
+	_, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, ErrorMatches, `attribute "bar" of system username "foo": invalid scalar:.*`)
+}
+
+func (s *YamlSuite) TestSnapYamlSystemUsernamesParsingBadScopeNonString(c *C) {
+	y := []byte(`name: binary
+version: 1.0
+system-usernames:
+  foo:
+    scope: 10
+`)
+	_, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, ErrorMatches, `scope on system username "foo" is not a string \(found int\)`)
+}
+
+func (s *YamlSuite) TestSnapYamlLinks(c *C) {
+	yLinks := []byte(`name: my-snap
+version: 1.0
+
+links:
+ donations:
+   - https://donate.me
+ contact:
+   - mailto:me@toto.space
+   - https://toto.space
+ bug-url:
+   - https://github.com/webteam-space/toto.space/issues
+ website:
+   - https://toto.space
+ source-code:
+   - https://github.com/webteam-space/toto.space
+`)
+	info, err := snap.InfoFromSnapYaml(yLinks)
+	c.Assert(err, IsNil)
+	c.Check(info.Links(), DeepEquals, map[string][]string{
+		"bug-url":     {"https://github.com/webteam-space/toto.space/issues"},
+		"contact":     {"mailto:me@toto.space", "https://toto.space"},
+		"donations":   {"https://donate.me"},
+		"source-code": {"https://github.com/webteam-space/toto.space"},
+		"website":     {"https://toto.space"},
+	})
+	c.Check(info.Contact(), Equals, "mailto:me@toto.space")
+}
+
+func (s *YamlSuite) TestSnapYamlEmptyLinksKey(c *C) {
+	yLinks := []byte(`name: my-snap
+version: 1.0
+
+links:
+ "":
+   - htps://toto.space
+`)
+	_, err := snap.InfoFromSnapYaml(yLinks)
+	c.Check(err, ErrorMatches, `links key cannot be empty`)
+}
+
+func (s *YamlSuite) TestSnapYamlInvalidLinksKey(c *C) {
+	yLinks := `name: my-snap
+version: 1.0
+
+links:
+ %s:
+    - link.website
+`
+
+	invalid := []string{
+		"--",
+		"1-2",
+		"aa-",
+	}
+
+	for _, k := range invalid {
+		_, err := snap.InfoFromSnapYaml([]byte(fmt.Sprintf(yLinks, k)))
+		c.Check(err, ErrorMatches, fmt.Sprintf(`links key is invalid: %s`, k))
+	}
+}
+
+func (s *YamlSuite) TestUnmarshalComponents(c *C) {
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+components:
+  test1:
+    type: standard
+    summary: test component
+    description: long component description
+  test2:
+    type: standard
+    summary: test component 2
+    description: long component description 2
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.InstanceName(), Equals, "snap")
+	c.Check(info.Components, DeepEquals, map[string]*snap.Component{
+		"test1": {
+			Type:        "standard",
+			Summary:     "test component",
+			Description: "long component description",
+			Name:        "test1",
+		},
+		"test2": {
+			Type:        "standard",
+			Summary:     "test component 2",
+			Description: "long component description 2",
+			Name:        "test2",
+		},
+	})
+}
+
+func (s *YamlSuite) TestUnmarshalComponentsHook(c *C) {
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+components:
+  test1:
+    type: standard
+    summary: test component
+    description: long component description
+    hooks:
+      install:
+        plugs: [network]
+        command-chain: [chain1, chain2]
+        environment:
+          k1: v1
+          k2: v2
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.InstanceName(), Equals, "snap")
+
+	component := info.Components["test1"]
+	c.Assert(component, NotNil)
+
+	hook := component.ExplicitHooks["install"]
+	c.Assert(hook, NotNil)
+	c.Check(hook.CommandChain, DeepEquals, []string{"chain1", "chain2"})
+	c.Check(hook.Environment, DeepEquals, *strutil.NewOrderedMap("k1", "v1", "k2", "v2"))
+}
+
+func (s *YamlSuite) TestUnmarshalComponentsHooksWithPlugs(c *C) {
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+components:
+  test1:
+    type: standard
+    summary: test component
+    description: long component description
+    hooks:
+      install:
+        plugs: [network, network-bind]
+      pre-refresh:
+        plugs: [test-plug]
+      post-refresh:
+      remove:
+  test2:
+    hooks:
+      pre-refresh:
+        plugs: [test-plug, test2-plug]
+plugs:
+  network-client:
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.InstanceName(), Equals, "snap")
+
+	type hook struct {
+		plugs []string
+	}
+
+	type component struct {
+		hooks map[string]hook
+	}
+
+	expectedComponents := map[string]component{
+		"test1": {
+			hooks: map[string]hook{
+				"install": {
+					plugs: []string{"network", "network-bind", "network-client"},
+				},
+				"pre-refresh": {
+					plugs: []string{"network-client", "test-plug"},
+				},
+				"post-refresh": {
+					plugs: []string{"network-client"},
+				},
+				"remove": {
+					plugs: []string{"network-client"},
+				},
+			},
+		},
+		"test2": {
+			hooks: map[string]hook{
+				"pre-refresh": {
+					plugs: []string{"network-client", "test-plug", "test2-plug"},
+				},
+			},
+		},
+	}
+
+	foundComponents := make(map[string]component, len(info.Components))
+	for _, foundComponent := range info.Components {
+		foundHooks := make(map[string]hook)
+		for _, foundHook := range foundComponent.ExplicitHooks {
+			c.Check(foundHook.Explicit, Equals, true)
+
+			foundPlugs := make([]string, 0, len(foundHook.Plugs))
+			for _, foundPlug := range foundHook.Plugs {
+				foundPlugs = append(foundPlugs, foundPlug.Name)
+			}
+
+			sort.Strings(foundPlugs)
+
+			foundHooks[foundHook.Name] = hook{
+				plugs: foundPlugs,
+			}
+		}
+
+		foundComponents[foundComponent.Name] = component{
+			hooks: foundHooks,
+		}
+	}
+
+	c.Check(foundComponents, DeepEquals, expectedComponents)
+}
+
+func (s *YamlSuite) TestUnmarshalComponentsHooksUnsupported(c *C) {
+	_, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+components:
+  test1:
+    type: standard
+    summary: test component
+    description: long component description
+    hooks:
+      unsupported:
+        plugs: [plug-for-unsupported]
+plugs:
+  network-client:
+`))
+	c.Assert(err, ErrorMatches, `unsupported component hook: "unsupported"`)
+}
+
+func (s *YamlSuite) TestUnmarshalComponentsError(c *C) {
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+components:
+  test1:
+    type: badtype
+`))
+	c.Assert(err.Error(), Equals, `cannot parse snap.yaml: unknown component type "badtype"`)
+	c.Assert(info, IsNil)
+}
+
+func (s *YamlSuite) TestUnmarshalComponentsHookSlotsError(c *C) {
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+components:
+  test1:
+    type: standard
+    hooks:
+      install:
+        slots: [slot-for-install]
+`))
+	c.Assert(err.Error(), Equals, `component hooks cannot have slots`)
+	c.Assert(info, IsNil)
+}
