@@ -24,7 +24,6 @@ import (
 	"bytes"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"os"
 	"os/exec"
 	"path"
@@ -228,7 +227,7 @@ func (s *Snap) Size() (size int64, err error) {
 }
 
 func (s *Snap) withUnpackedFile(filePath string, f func(p string) error) error {
-	tmpdir, err := ioutil.TempDir("", "read-file")
+	tmpdir, err := os.MkdirTemp("", "read-file")
 	if err != nil {
 		return err
 	}
@@ -264,7 +263,7 @@ func (s *Snap) RandomAccessFile(filePath string) (interface {
 // ReadFile returns the content of a single file inside a squashfs snap.
 func (s *Snap) ReadFile(filePath string) (content []byte, err error) {
 	err = s.withUnpackedFile(filePath, func(p string) (err error) {
-		content, err = ioutil.ReadFile(p)
+		content, err = os.ReadFile(p)
 		return
 	})
 	if err != nil {
@@ -273,6 +272,41 @@ func (s *Snap) ReadFile(filePath string) (content []byte, err error) {
 	return content, nil
 }
 
+func (s *Snap) ReadLink(filePath string) (string, error) {
+	// XXX: This could be optimized by reading a cached version of
+	// unsquashfs raw output where the symlink's target is available.
+	// Check -> func fromRaw(raw []byte) (*stat, error)
+	var target string
+	err := s.withUnpackedFile(filePath, func(p string) (err error) {
+		target, err = os.Readlink(p)
+		return err
+	})
+	if err != nil {
+		return "", err
+	}
+	return target, nil
+}
+
+func (s *Snap) Lstat(filePath string) (os.FileInfo, error) {
+	var fileInfo os.FileInfo
+
+	err := s.Walk(filePath, func(path string, info os.FileInfo, err error) error {
+		if filePath == path {
+			fileInfo = info
+		}
+		return err
+	})
+	if err != nil {
+		return nil, err
+	}
+
+	if fileInfo == nil {
+		return nil, os.ErrNotExist
+	}
+
+	return fileInfo, nil
+}
+
 // skipper is used to track directories that should be skipped
 //
 // Given sk := make(skipper), if you sk.Add("foo/bar"), then
@@ -299,9 +333,10 @@ func (sk skipper) Has(path string) bool {
 }
 
 // pre-4.5 unsquashfs writes a funny header like:
-//     "Parallel unsquashfs: Using 1 processor"
-//     "1 inodes (1 blocks) to write"
-//     ""   <-- empty line
+//
+//	"Parallel unsquashfs: Using 1 processor"
+//	"1 inodes (1 blocks) to write"
+//	""   <-- empty line
 var maybeHeaderRegex = regexp.MustCompile(`^(Parallel unsquashfs: Using .* processor.*|[0-9]+ inodes .* to write)$`)
 
 // Walk (part of snap.Container) is like filepath.Walk, without the ordering guarantee.
@@ -352,10 +387,14 @@ func (s *Snap) Walk(relative string, walkFn filepath.WalkFunc) error {
 				return err
 			}
 		} else {
-			path := filepath.Join(relative, st.Path())
+			path := filepath.Join(".", st.Path())
 			if skipper.Has(path) {
 				continue
 			}
+			// skip if path is not under given relative path
+			if relative != "." && !strings.HasPrefix(path, relative) {
+				continue
+			}
 			err = walkFn(path, st, nil)
 			if err != nil {
 				if err == filepath.SkipDir && st.IsDir() {
@@ -379,10 +418,10 @@ func (s *Snap) Walk(relative string, walkFn filepath.WalkFunc) error {
 
 // ListDir returns the content of a single directory inside a squashfs snap.
 func (s *Snap) ListDir(dirPath string) ([]string, error) {
-	output, err := exec.Command(
-		"unsquashfs", "-no-progress", "-dest", "_", "-l", s.path, dirPath).CombinedOutput()
+	output, stderr, err := osutil.RunSplitOutput(
+		"unsquashfs", "-no-progress", "-dest", "_", "-l", s.path, dirPath)
 	if err != nil {
-		return nil, osutil.OutputErr(output, err)
+		return nil, osutil.OutputErrCombine(output, stderr, err)
 	}
 
 	prefixPath := path.Join("_", dirPath)
@@ -431,9 +470,7 @@ func (e *errPathsNotReadable) Error() string {
 
 	b.WriteString("cannot access the following locations in the snap source directory:\n")
 	for _, p := range e.paths {
-		fmt.Fprintf(&b, "- ")
-		fmt.Fprintf(&b, p)
-		fmt.Fprintf(&b, "\n")
+		fmt.Fprintf(&b, "- %s\n", p)
 	}
 	if len(e.paths) == maxErrPaths {
 		fmt.Fprintf(&b, "- too many errors, listing first %v entries\n", maxErrPaths)
@@ -504,6 +541,14 @@ type BuildOpts struct {
 	ExcludeFiles []string
 }
 
+// MinimumSnapSize is the smallest size a snap can be. The kernel attempts to read a
+// partition table from the snap when a loopback device is created from it. If the snap
+// is smaller than this size, some versions of the kernel will print error logs while
+// scanning the loopback device for partitions.
+// TODO: revisit if necessary, some distros (eg. openSUSE) patch squashfs-tools to pad to 64k but
+// kernel should work with this
+const MinimumSnapSize int64 = 16384
+
 // Build builds the snap.
 func (s *Snap) Build(sourceDir string, opts *BuildOpts) error {
 	if opts == nil {
@@ -536,6 +581,7 @@ func (s *Snap) Build(sourceDir string, opts *BuildOpts) error {
 		"-no-fragments",
 		"-no-progress",
 	)
+
 	if len(opts.ExcludeFiles) > 0 {
 		cmd.Args = append(cmd.Args, "-wildcards")
 		for _, excludeFile := range opts.ExcludeFiles {
@@ -543,11 +589,15 @@ func (s *Snap) Build(sourceDir string, opts *BuildOpts) error {
 		}
 	}
 	snapType := opts.SnapType
-	if snapType != "os" && snapType != "core" && snapType != "base" {
+	switch snapType {
+	case "os", "core", "base", "snapd":
+		// -xattrs is default, but let's be explicit about it
+		cmd.Args = append(cmd.Args, "-xattrs")
+	default:
 		cmd.Args = append(cmd.Args, "-all-root", "-no-xattrs")
 	}
 
-	return osutil.ChDir(sourceDir, func() error {
+	err = osutil.ChDir(sourceDir, func() error {
 		output, err := cmd.CombinedOutput()
 		if err != nil {
 			return MksquashfsError{fmt.Sprintf("mksquashfs call failed: %s", osutil.OutputErr(output, err))}
@@ -555,6 +605,13 @@ func (s *Snap) Build(sourceDir string, opts *BuildOpts) error {
 
 		return nil
 	})
+	if err != nil {
+		return err
+	}
+
+	// Grow the snap if it is smaller than the minimum snap size. See
+	// MinimumSnapSize for more details.
+	return s.growSnapToMinSize(MinimumSnapSize)
 }
 
 // BuildDate returns the "Creation or last append time" as reported by unsquashfs.
@@ -562,6 +619,21 @@ func (s *Snap) BuildDate() time.Time {
 	return BuildDate(s.path)
 }
 
+func (s *Snap) growSnapToMinSize(minSize int64) error {
+	size, err := s.Size()
+	if err != nil {
+		return fmt.Errorf("cannot get size of snap: %w", err)
+	}
+	if size >= minSize {
+		return nil
+	}
+	if err := os.Truncate(s.path, minSize); err != nil {
+		return fmt.Errorf("cannot grow snap to minimum size: %w", err)
+	}
+
+	return nil
+}
+
 // BuildDate returns the "Creation or last append time" as reported by unsquashfs.
 func BuildDate(path string) time.Time {
 	var t0 time.Time
