@@ -531,6 +531,7 @@ pkg_dependencies_ubuntu_generic(){
         libglib2.0-dev
         libseccomp-dev
         libudev-dev
+        lsof
         man
         mtools
         netcat-openbsd
@@ -544,6 +545,23 @@ pkg_dependencies_ubuntu_generic(){
         "
 }
 
+pkg_dependencies_ubuntu_nested(){
+    echo "
+        ca-certificates
+        cloud-image-utils
+        genisoimage
+        kpartx
+        mtools
+        ovmf
+        qemu-kvm
+        qemu-utils
+        snapd
+        sshpass
+        xdelta3
+        xz-utils
+        "
+}
+
 pkg_dependencies_ubuntu_classic(){
     echo "
         avahi-daemon
@@ -551,14 +569,11 @@ pkg_dependencies_ubuntu_classic(){
         fish
         fontconfig
         gnome-keyring
-        jq
-        man
         nfs-kernel-server
         printer-driver-cups-pdf
         python3-dbus
         python3-gi
         python3-yaml
-        upower
         weston
         xdg-user-dirs
         xdg-utils
@@ -706,6 +721,7 @@ pkg_dependencies_fedora_centos_common(){
         jq
         iptables
         iptables-services
+        lsof
         man
         net-tools
         nmap-ncat
@@ -723,18 +739,14 @@ pkg_dependencies_fedora_centos_common(){
         strace
         zsh
         "
-    if ! os.query is-centos 9; then
-        echo "
-            fish
-            redhat-lsb-core
-        "
-    fi
 }
 
 pkg_dependencies_fedora(){
     echo "
-         libcap-static
-        "
+        fish
+        libcap-static
+        script
+    "
 }
 
 pkg_dependencies_amazon(){
@@ -764,6 +776,7 @@ pkg_dependencies_amazon(){
         jq
         iptables-services
         libcap-static
+        lsof
         man
         nc
         net-tools
@@ -798,14 +811,17 @@ pkg_dependencies_opensuse(){
         iptables
         jq
         lsb-release
+        lsof
         man
         man-pages
         nfs-kernel-server
         nss-mdns
         osc
         PackageKit
+        procps
         python3-yaml
         strace
+        sysvinit-tools
         netcat-openbsd
         rpm-build
         udisks2
@@ -814,6 +830,7 @@ pkg_dependencies_opensuse(){
         xdg-user-dirs
         xdg-utils
         zsh
+        libcap-progs
         "
     if os.query is-opensuse tumbleweed; then
         echo "
@@ -843,6 +860,7 @@ pkg_dependencies_arch(){
     libseccomp
     libcap
     libx11
+    lsof
     man
     net-tools
     nfs-utils
@@ -872,8 +890,13 @@ pkg_dependencies(){
             pkg_dependencies_ubuntu_core
             ;;
         ubuntu-*|debian-*)
-            pkg_dependencies_ubuntu_generic
-            pkg_dependencies_ubuntu_classic
+            if tests.nested is-nested &>/dev/null; then
+                pkg_dependencies_ubuntu_generic
+                pkg_dependencies_ubuntu_nested
+            else
+                pkg_dependencies_ubuntu_generic
+                pkg_dependencies_ubuntu_classic
+            fi
             ;;
         amazon-*)
             pkg_dependencies_amazon
