@@ -53,11 +53,17 @@ create_test_user(){
     fi
     unset owner
 
-    # Add a new line first to prevent an error which happens when
-    # the file has not new line, and we see this:
-    # syntax error, unexpected WORD, expecting END or ':' or '\n'
-    echo >> /etc/sudoers
-    echo 'test ALL=(ALL) NOPASSWD:ALL' >> /etc/sudoers
+    if [ -d /etc/sudoers.d ]; then
+        # this also works around systems where /etc/sudo is empty and /etc could
+        # be ephemeral, eg. openSUSE Tumbleweed which keeps /usr/etc/sudoers
+        echo 'test ALL=(ALL) NOPASSWD:ALL' >> /etc/sudoers.d/99-test-user
+    else
+        # Add a new line first to prevent an error which happens when
+        # the file has not new line, and we see this:
+        # syntax error, unexpected WORD, expecting END or ':' or '\n'
+        echo >> /etc/sudoers
+        echo 'test ALL=(ALL) NOPASSWD:ALL' >> /etc/sudoers
+    fi
 
     chown test:test -R "$SPREAD_PATH"
     chown test:test "$SPREAD_PATH/../"
@@ -156,37 +162,17 @@ build_arch_pkg() {
     base_version="$(head -1 debian/changelog | awk -F '[()]' '{print $2}')"
     version="1337.$base_version"
     packaging_path=packaging/arch
-    archive_name=snapd-$version.tar
 
     rm -rf /tmp/pkg
-    mkdir -p /tmp/pkg/sources/snapd
-    cp -ra -- * /tmp/pkg/sources/snapd/
+    mkdir -p /tmp/pkg/
+    cp -av "$packaging_path"/* /tmp/pkg
 
     # shellcheck disable=SC2086
-    tar -C /tmp/pkg/sources -cf "/tmp/pkg/$archive_name" "snapd"
-    cp "$packaging_path"/* "/tmp/pkg"
+    ./packaging/pack-source -v "$version" -o "/tmp/pkg/" -s
 
-    # fixup PKGBUILD which builds a package named snapd-git with dynamic version
-    #  - update pkgname to use snapd
-    #  - kill dynamic version
-    #  - packaging functions are named package_<pkgname>(), update it to package_snapd()
-    #  - update source path to point to local archive instead of git
-    #  - fix package version to $version
     sed -i \
-        -e "s/^source=.*/source=(\"$archive_name\")/" \
-        -e "s/pkgname=snapd.*/pkgname=snapd/" \
         -e "s/pkgver=.*/pkgver=$version/" \
-        -e "s/package_snapd-git()/package_snapd()/" \
         /tmp/pkg/PKGBUILD
-    # comment out automatic package version update block `pkgver() { ... }` as
-    # it's only useful when building the package manually
-    awk '
-    /BEGIN/ { strip = 0; last = 0 }
-    /pkgver\(\)/ { strip = 1 }
-    /^}/ { if (strip) last = 1 }
-    // { if (strip) { print "#" $0; if (last) { last = 0; strip = 0}} else { print $0}}
-    ' < /tmp/pkg/PKGBUILD > /tmp/pkg/PKGBUILD.tmp
-    mv /tmp/pkg/PKGBUILD.tmp /tmp/pkg/PKGBUILD
 
     chown -R test:test /tmp/pkg
     unshare -n -- \
@@ -232,16 +218,6 @@ install_dependencies_gce_bucket(){
 ###
 
 prepare_project() {
-    # we install chrony on xenial (as its also used in later distros), so the
-    # ntp service was in conflict, degrading the systemd unit and sometimes breaking
-    # NTP syncs
-    if os.query is-xenial; then
-      systemctl stop ntp.service
-      systemctl disable ntp.service
-      apt-get remove --purge -y ntp
-      systemctl reset-failed
-    fi
-
     if os.query is-ubuntu && os.query is-classic; then
         apt-get remove --purge -y lxd lxcfs || true
         apt-get autoremove --purge -y
@@ -640,6 +616,7 @@ prepare_project_each() {
 prepare_suite() {
     # shellcheck source=tests/lib/prepare.sh
     . "$TESTSLIB"/prepare.sh
+
     # os.query cannot be used because first time the suite is prepared, the current system
     # is classic ubuntu, so it is needed to check the system set in $SPREAD_SYSTEM
     if is_test_target_core; then
@@ -648,6 +625,10 @@ prepare_suite() {
         prepare_classic
     fi
 
+    if [ -n "$TAG_FEATURES" ]; then
+        snap set system journal.persistent=true
+    fi
+
     # Make sure the suite starts with a clean environment and with the snapd state restored
     # shellcheck source=tests/lib/reset.sh
     "$TESTSLIB"/reset.sh --reuse-core
@@ -670,16 +651,6 @@ prepare_suite_each() {
 
     # Save all the installed packages
     if os.query is-classic; then
-        # lxd-installer is in cloud images starting from 24.04. This package
-        # installs lxd when any lxc command is run. This caused problems
-        # because if we install snapcraft & lxd, in the restore step lxd is
-        # removed, and after that snapcraft is removed. However, snapcraft's
-        # remove hook calls lxd and triggers a new installation of lxd, and in
-        # turn when we try to remove core22, it fails as lxd has been
-        # re-installed and depends on that base. Therefore, we remove it to
-        # prevent these issues, and we do that before we get the list of
-        # installed packages to make sure we do not re-install it again.
-        apt remove -y --purge lxd-installer || true
         tests.pkgs list-installed > installed-initial.pkgs
     fi
 
@@ -733,6 +704,10 @@ prepare_suite_each() {
 }
 
 restore_suite_each() {
+    if not tests.nested is-nested; then
+        "$TESTSLIB"/collect-artifacts.sh features --after-non-nested-task
+        "$TESTSLIB"/collect-artifacts.sh locks
+    fi
     local variant="$1"
 
     rm -f "$RUNTIME_STATE_PATH/audit-stamp"
@@ -816,6 +791,8 @@ restore_suite() {
         # shellcheck source=tests/lib/pkgdb.sh
         . "$TESTSLIB"/pkgdb.sh
         distro_purge_package snapd
+        # On Tumbleweed, removing the package doesn't stop the snapd units so ensure they are stopped
+        systemctl stop snapd.socket snapd.service || true
         if [[ "$SPREAD_SYSTEM" != opensuse-* && "$SPREAD_SYSTEM" != arch-* ]]; then
             # A snap-confine package never existed on openSUSE or Arch
             distro_purge_package snap-confine
