@@ -87,7 +87,47 @@ disable_refreshes() {
     snap refresh --time --abs-time | MATCH "last: 2[0-9]{3}"
 }
 
+setup_snapd_proxy() {
+    if [ "${SNAPD_USE_PROXY:-}" != true ]; then
+        return
+    fi
+
+    mkdir -p /etc/systemd/system/snapd.service.d
+    cat <<EOF > /etc/systemd/system/snapd.service.d/proxy.conf
+[Service]
+Environment="HTTPS_PROXY=$HTTPS_PROXY HTTP_PROXY=$HTTP_PROXY https_proxy=$HTTPS_PROXY http_proxy=$HTTP_PROXY NO_PROXY=$NO_PROXY no_proxy=$NO_PROXY"
+EOF
+
+    # We change the service configuration so reload and restart
+    # the units to get them applied
+    systemctl daemon-reload
+    # restart the service (it pulls up the socket)
+    systemctl restart snapd.service
+}
+
+setup_system_proxy() {
+    mkdir -p "$SNAPD_WORK_DIR"
+    if [ "${SNAPD_USE_PROXY:-}" = true ]; then    
+        cp -f /etc/environment "$SNAPD_WORK_DIR"/environment.bak
+        {
+            echo "HTTPS_PROXY=$HTTPS_PROXY"
+            echo "HTTP_PROXY=$HTTP_PROXY"
+            echo "https_proxy=$HTTPS_PROXY"
+            echo "http_proxy=$HTTP_PROXY"
+            echo "NO_PROXY=$NO_PROXY"
+            echo "no_proxy=$NO_PROXY"
+        } >> /etc/environment
+    fi
+}
+
 setup_systemd_snapd_overrides() {
+    local burst
+    burst=10
+    if [ "$SPREAD_BACKEND" = "garden" ]; then
+        # the tests execute much faster and the repeated stop/start of snapd in
+        # prepare may eventually go over the start limit
+        burst=30
+    fi
     mkdir -p /etc/systemd/system/snapd.service.d
     cat <<EOF > /etc/systemd/system/snapd.service.d/local.conf
 [Service]
@@ -97,7 +137,16 @@ ExecStartPre=/bin/touch /dev/iio:device0
 [Unit]
 # The default limit is usually 5, which can be easily hit in 
 # a fast system with few systemd units
-StartLimitBurst=10
+StartLimitBurst=${burst}
+StartLimitIntervalSec=10s
+EOF
+
+    mkdir -p /etc/systemd/system/snapd.socket.d
+    cat <<EOF > /etc/systemd/system/snapd.socket.d/local.conf
+[Unit]
+# The default limit is usually 5, which can be easily hit in
+# a fast system with few systemd units
+StartLimitBurst=${burst}
 StartLimitIntervalSec=10s
 EOF
 
@@ -354,6 +403,9 @@ prepare_each_core() {
 }
 
 prepare_classic() {
+    # Configure the proxy in the system when it is required
+    setup_system_proxy   
+
     # Skip building snapd when REUSE_SNAPD is set to 1
     if [ "$REUSE_SNAPD" != 1 ]; then
         distro_install_build_snapd
@@ -412,6 +464,16 @@ prepare_classic() {
     fi
     snap list snapd
 
+    setup_snapd_proxy
+
+    mount_dir="$(os.paths snap-mount-dir)"
+    if ! getcap "$mount_dir"/snapd/current/usr/lib/snapd/snap-confine | grep "cap_sys_admin"; then
+        echo "snapd snap is missing file capabilities on snap-confine"
+        echo "and is not usable"
+        echo "ensure it has been correctly built (wipe snapcraft containers and rebuild)"
+        exit 1
+    fi
+
     setup_systemd_snapd_overrides
 
     if [ "$REMOTE_STORE" = staging ]; then
@@ -463,6 +525,7 @@ prepare_classic() {
 
         prepare_reexec_override
         prepare_state_lock "SNAPD PROJECT"
+        prepare_tag_features
         prepare_memory_limit_override
         disable_refreshes
 
@@ -478,6 +541,28 @@ prepare_classic() {
             fi
         fi
 
+        # lxd-installer is in cloud images starting from 24.04. This package
+        # installs lxd when any lxc command is run. This caused problems
+        # because if we install snapcraft & lxd, in the restore step lxd is
+        # removed, and after that snapcraft is removed. However, snapcraft's
+        # remove hook calls lxd and triggers a new installation of lxd, and in
+        # turn when we try to remove core22, it fails as lxd has been
+        # re-installed and depends on that base. Therefore, we remove it to
+        # prevent these issues, and we do that before we get the list of
+        # installed packages to make sure we do not re-install it again.
+        if ( os.query is-ubuntu || os.query is-debian ) && tests.pkgs is-installed lxd-installer; then
+            extra=
+            if os.query is-ubuntu-ge 25.10; then
+                # the following dependency is in place in 25.10:
+                # ubuntu-server:amd64 Depends lxd-installer
+                #
+                # NOTE: this will leave some packages without explicit
+                # dependency pulling them in
+                extra=ubuntu-server
+            fi
+            apt remove -y --purge lxd-installer $extra
+        fi
+
         setup_experimental_features
 
         systemctl stop snapd.{service,socket}
@@ -967,13 +1052,14 @@ EOF
 
     # remove the kernel module from the kernel snap
     rm "$module_path"
-    # depmod wants a lib subdir, fake it and remove after invocation
-    mkdir pc-kernel/lib
+    # depmod wants a lib subdir
+    mkdir -p pc-kernel/lib
     ln -s ../modules pc-kernel/lib/modules
     depmod -b pc-kernel/ "$kern_ver"
-    rm -rf pc-kernel/lib
     # append component meta-information
-    printf 'components:\n  %s:\n    type: kernel-modules\n' "$comp_name" >> pc-kernel/meta/snap.yaml
+    #shellcheck disable=SC2016
+    gojq --arg COMP_NAME "${comp_name}" '.components = {$COMP_NAME:{"type":"kernel-modules"}}' --yaml-input pc-kernel/meta/snap.yaml --yaml-output >pc-kernel/meta/snap.yaml.new
+    mv pc-kernel/meta/snap.yaml.new pc-kernel/meta/snap.yaml
 }
 
 uc24_build_initramfs_kernel_snap() {
@@ -993,6 +1079,8 @@ uc24_build_initramfs_kernel_snap() {
     ubuntu-core-initramfs create-initrd --kernelver="$kernelver" --kerneldir pc-kernel/modules/"$kernelver" \
                           --firmwaredir pc-kernel/firmware --output initrd.img
 
+    # Check that manifest is generated
+    stat manifest-initramfs.yaml-"$kernelver"
     initrd_f=initrd.img-"$kernelver"
     unmkinitramfs "$initrd_f" initrd
 
@@ -1026,6 +1114,14 @@ uc24_build_initramfs_kernel_snap() {
         cp -a ./extra-kernel-snap/* ./pc-kernel
     fi
 
+    if [ -n "${NESTED_KERNEL_REMOVE_COMPONENTS-}" ]; then
+        #shellcheck disable=SC2016
+        gojq 'del(.components)' --yaml-input pc-kernel/meta/snap.yaml --yaml-output >pc-kernel/meta/snap.yaml.new
+        mv pc-kernel/meta/snap.yaml.new pc-kernel/meta/snap.yaml
+        gojq 'del(.slots)' --yaml-input pc-kernel/meta/snap.yaml --yaml-output >pc-kernel/meta/snap.yaml.new
+        mv pc-kernel/meta/snap.yaml.new pc-kernel/meta/snap.yaml
+    fi
+
     if [ -n "$NESTED_KERNEL_MODULES_COMP" ] && is_test_target_core_ge 24; then
         # "split" kernel in kernel-modules component and kernel
         move_module_to_component "$NESTED_COMP_KERNEL_MODULE_NAME" "$NESTED_KERNEL_MODULES_COMP"
@@ -1237,7 +1333,12 @@ setup_reflash_magic() {
         fi
     elif is_test_target_core 24; then
         build_snapd_snap_with_run_mode_firstboot_tweaks "$IMAGE_HOME"
-        cp "$TESTSLIB/assertions/ubuntu-core-24-amd64.model" "$IMAGE_HOME/pc.model"
+        if os.query is-arm; then
+            cp "$TESTSLIB/assertions/ubuntu-core-24-arm64.model" "$IMAGE_HOME/pc.model"
+        else
+            cp "$TESTSLIB/assertions/ubuntu-core-24-amd64.model" "$IMAGE_HOME/pc.model"
+        fi
+        
     else
         # FIXME: install would be better but we don't have dpkg on
         #        the image
@@ -1338,7 +1439,7 @@ EOF
         # so for now, don't include snapd.debug=1, but eventually it would be
         # nice to have this on
 
-        if [[ "$SPREAD_BACKEND" =~ google ]]; then
+        if [[ "$SPREAD_BACKEND" =~ google ]] || [[ "$SPREAD_BACKEND" =~ openstack ]]; then
             # the default console settings for snapd aren't super useful in GCE,
             # instead it's more useful to have all console go to ttyS0 which we 
             # can read more easily than tty1 for example
@@ -1436,7 +1537,7 @@ EOF
                     $EXTRA_FUNDAMENTAL \
                     --snap "${extra_snap[0]}" \
                     --output-dir "$IMAGE_HOME"
-    rm -f ./pc-kernel_*.{snap,assert} ./pc-kernel.{snap,assert} ./pc_*.{snap,assert} ./snapd_*.{snap,assert} ./core{20,22}.{snap,assert}
+    rm -f ./pc-kernel_*.{snap,assert} ./pc-kernel.{snap,assert} ./pc_*.{snap,assert} ./snapd_*.{snap,assert} ./core{20,22,24}.{snap,assert}
 
     if os.query is-arm; then
         LOOP_PARTITION=1
@@ -1597,13 +1698,52 @@ EOF
     fi
 }
 
+prepare_tag_features(){
+    CONF_FILE="/etc/systemd/system/snapd.service.d/99-feature-tags.conf"
+    RESTART=false
+
+    if [ -n "$TAG_FEATURES" ]; then
+        # Generate the config file when it does not exist and when the threshold has changed different
+        if ! [ -f "$CONF_FILE" ]; then
+            cat <<EOF > "$CONF_FILE"
+[Service]
+Environment=SNAPPY_TESTING=1
+Environment=SNAPD_TRACE=1
+Environment=SNAPD_JSON_LOGGING=1
+EOF
+            RESTART=true
+        fi
+    elif [ -f "$CONF_FILE" ]; then
+        rm -f "$CONF_FILE"
+        RESTART=true
+    fi
+
+    if [ "$RESTART" = "true" ]; then
+        # the service setting may have changed in the service so we need
+        # to ensure snapd is reloaded
+        systemctl daemon-reload
+        systemctl restart snapd
+    fi
+}
+
 # prepare_ubuntu_core will prepare ubuntu-core 16+
 prepare_ubuntu_core() {
+    # Configure the proxy in the system when it is required
+    setup_system_proxy
+
     # we are still a "classic" image, prepare the surgery
     if [ -e /var/lib/dpkg/status ]; then
         setup_reflash_magic
         REBOOT
     fi
+    setup_snapd_proxy
+
+    # We setup the ntp server in case it is defined in the current env
+    # This is not needed in classic systems becuase the images already have ntp configured
+    if [ -n "${NTP_SERVER:-}" ]; then
+        sed -i "s/^#NTP=.*/NTP=$NTP_SERVER/" /etc/systemd/timesyncd.conf
+        systemctl restart systemd-timesyncd
+    fi
 
     disable_journald_rate_limiting
     disable_journald_start_limiting
@@ -1656,6 +1796,8 @@ prepare_ubuntu_core() {
             rsync_snap="test-snapd-rsync-core22"
         elif os.query is-core24; then
             rsync_snap="test-snapd-rsync-core24"
+        elif os.query is-core26; then
+            rsync_snap="test-snapd-rsync-core26"
         fi
         snap install --devmode --edge "$rsync_snap"
         snap alias "$rsync_snap".rsync rsync
@@ -1686,6 +1828,9 @@ prepare_ubuntu_core() {
         if os.query is-core24; then
             cache_snaps test-snapd-sh-core24
         fi
+        if os.query is-core26; then
+            cache_snaps test-snapd-sh-core26
+        fi
     fi
 
     disable_refreshes
@@ -1711,6 +1856,7 @@ prepare_ubuntu_core() {
         remove_disabled_snaps
         prepare_memory_limit_override
         prepare_state_lock "SNAPD PROJECT"
+        prepare_tag_features
         setup_experimental_features
         systemctl stop snapd.service snapd.socket
         save_snapd_state
